// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: CRAPS.FTH      Author: Oliver SINGLA	 Date: Dec,20,94    Version: 1.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 Craps game.												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;

( Make SCREEN the context vocabulary )
screen

( Global variables )
int loot						       // Amount of money available
int bet							       // Amount of the bet


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : doTittle												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Tittle													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  doTittle
   dup $len const len
   #cols  len 4 + -  2/	 const  x
   x 0 ' !xychar  x 1+ 0 len 2+ ' !xynchar  x len + 3 + 0 ' !xychar
   x 1 "   " !xystring  x 2+ 1 rot !xystring  x len + 2+ 1 "  " !xystring
   x 2 ' !xychar  x 1+ 2 len 2+ ' !xynchar  x len + 3 + 2 ' !xychar
   ;   // doTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : tittle													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : craps													        \\
// 	 Calls : doTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Tittle
   80 fstring tittle$
   " FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " NEVADA CRAPS GAME" +$ doTittle
   ;   // Tittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Rules													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the rules of the games.										        \\
//    Callers : craps													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Rules
    0  4 " This game simulates the game of ~{2}craps~{0} played according to standard Nevada ~{2}craps"  !xylstring
    0  5 " table rules. That is:" !xylstring
    0  6 " - ~{1}2~{0}, ~{1}3~{0} or ~{1}12~{0} on the first throw : you loose;" !xylstring
    0  7 " - ~{1}7~{0} or ~{1}11~{0} on the first throw    : you win;" !xylstring
    0  8 " - any other value on the first throw becomes your 'point'. You continue to" !xylstring
    0  9 "   thrown the dices until you win or you loose:" !xylstring
    0 10 "     + on ~{1}7~{0}          : you loose," !xylstring
    0 11 "     + on your point : you win." !xylstring
   ;   // Rules


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitGame												        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize the game.											        \\
//    Callers : craps													        \\
// 	 Calls : loot													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InitGame
   100 loot !						       // Amount of money available
   ;   // InitGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : loot.													        \\
// 	 Stack : ( --- )												        \\
// Description : Show the amount of money available.									        \\
//    Callers : Diag1stThrow:2 PlayAGame										        \\
// 	 Calls : loot													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  loot.
   80 fstring line$
   " You now have ~{1}" line$ $!
   loot @ <# #S #> line$ $+
   " $~{0}." +$
    0 13 rot  !xylstring clreol
   ;   // loot.


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : WhatIsYourBet?												        \\
// 	 Stack : ( --- n )												        \\
// Description : Ask for the amount of the bet.										        \\
//    Callers : PlayAGame												        \\
// 	 Calls : loot													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  WhatIsYourBet?
   6 fstring bet$
   80 fstring msg$
   20 13 " What is your bet ? " !xylstring
   wherex const x
   wherey const y
   logical done?
   begin
       x y gotoxy  bet$ $mlen spaces  x y gotoxy
       bet$ dup $mlen $input
       done? true!
       bet$ $len 0== if
	   " Enter a bet !" msg$ $!
	   done? false!
       endif
       bet$ 0 $ConvNb dup -1 == if
	   drop bet$ msg$ $!
	   "  invalid number !" msg$ $+ drop
	   done? false!
       endif
       done? true? if
	   swap bet$ $len != if
	       drop bet$ msg$ $!
	       "  not a valid number !" msg$ $+ drop
	       done? false!
	   endif
       endif
       done? true? if
	   dup loot @ > if
	       drop bet$ msg$ $!
	       "  not enought money !" msg$ $+ drop
	       done? false!
	   endif
       endif
       done? false? if
	   46 13 msg$ !xylstring clreol
       else
	   46 13 gotoxy clreol
       endif
       done? true?
   until
   ;   // WhatIsYourBet?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ThrowDice												        \\
// 	 Stack : ( n1 addrz --- n2 )											        \\
// Description : Throw the dice.											        \\
//    Callers : PlayAGame												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ThrowDice
   swap const y
   ?string
    0 y rot !xylstring space
   wherex const x
   35 y " Press a key to stop..." !xylstring
   ?UpdateCursor false!
   int dice#1
   int dice#2
   20 fstring throw$
   begin
       1 6 rnd dice#1 !
       1 6 rnd dice#2 !
       dice#1 @ <#S> throw$ $!
       dice#2 @ <# # 0x20 hold #> throw$ $+
       "  -> " +$
       dice#1 @ dice#2 @ + <#S> +$
       "  " +$
       x y rot !xystring
       ?key
   until
   key drop
   35 y gotoxy clreol
   ?UpdateCursor true!
   dice#1 @ dice#2 @ +
   ;   // ThrowDice


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Diag1stThrow												        \\
// 	 Stack : ( n --- n ff )												        \\
// 	    or : ( n --- tf )												        \\
// Description : Diagnostic on the first throw.										        \\
//    Callers : PlayAGame												        \\
// 	 Calls : loot:2 bet:2 loot.:2											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Diag1stThrow
   const n
   n 2 ==  n 3 == or  n 12 == or if
       TRUE " You loose !"
       bet @ loot -!  loot.
   else
       n 7 ==  n 11 == or if
	   TRUE " You win !"
	   bet @ loot +!  loot.
       else
	   n FALSE " This is your point."
       endif
   endif
   35 15 rot !xylstring
   ;   // Diag1stThrow


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : RollForPoint												        \\
// 	 Stack : ( n --- )												        \\
// Description : Throw dice until loose or point.									        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  RollForPoint
   const point
   begin
       16 " Next throw  : " ThrowDice                          // ( --- n )
       const n
       n 7 == if
	   35 16 " You loose !" !xylstring
	   bet @ loot -!  loot.
	   TRUE
       else
	   n point == if
	       35 16 " You win !" !xylstring
	       bet @ loot +!  loot.
	       TRUE
	   else
	       35 16 " Neither 7, nor your point. Press a key" !xylstring
	       key drop 35 16 gotoxy clreol
	       FALSE
	   endif
       endif
   until
   ;   // RollForPoint


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PlayAGame												        \\
// 	 Stack : ( --- )												        \\
// Description : Play a game of craps.											        \\
//    Callers : craps													        \\
// 	 Calls : bet loot. WhatIsYourBet? ThrowDice Diag1stThrow							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PlayAGame
   int answer
   begin
       loot.						       // Show the amount of available money
       WhatIsYourBet? bet !				       // Ask for the bet
       15 " First throw : " ThrowDice                          // ( --- n ) 1st throw
       Diag1stThrow					       // ( n --- n ff ) or ( n --- tf )
       not if  RollForPoint  endif
       loot @0== if
	   'N answer C!
	   0 18 " Sorry, no more money..." !xylstring
       else
	   0 18 " Press ~{1}Y~{0} to play again or ~{1}N~{0} to stop " !xylstring
	   begin
	       key CharUppercase answer C!
	       answer C@ 'Y ==  answer C@ 'N ==	 or
	   until
	   18 ClrLine
       endif
       15 ClrLine
       16 ClrLine
       answer C@ 'N ==
   until
   ;   // PlayAGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : craps													        \\
// 	 Stack : ( --- )												        \\
// Description : The craps game.											        \\
//    Callers : g													        \\
// 	 Calls : Tittle Rules InitGame PlayAGame									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  craps
   GetVideoMode const prevVideoMode			       // Save current video mode
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   '~ >xylstring_char
   0 black lightgray >xylstring_attr
   1 black white     >xylstring_attr
   2 black yellow    >xylstring_attr
   Tittle
   Rules
   InitGame						       // Initialize the game
   PlayAGame						       // Play a game
   prevVideoMode GetVideoMode != if			       // Restore previous video mode ?
       prevVideoMode SetVideoMode			       // Restore previous video mode
   endif
   ;   // craps


( Print informations about Code/Data size )
( --------------------------------------- )

infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

\ " \\froth\\examples\\craps.exe" save.exe
\ cr " CRAPS.EXE app size is " type . "  bytes" type

