// ---------------------------------------------------------------------------------------------------------------------------- \\
//                                                                                                                              \\
// File: CUSTOMER.FTH   Author: O.S.             Date: Oct-10-94    Version: 0.01                                               \\
// ----                 ------                   ----               -------                                                     \\
//                                                                                                                              \\
// Description: Using a customers file.                                                                                         \\
//                                                                                                                              \\
//                                                                                                                              \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

decimal                                                        // Force decimal mode for integer numeric I/O

( Forget all previous definitions )
( ------------------------------- )

forget <task>
: <task> ;

screen                                                         // Access to screen definitions


( Colors used into the application )
( -------------------------------- )

black LightGray BackForeGround constant C_NormalText           // Normal text


( Handle on customers opened file )
( ------------------------------- )

integer ghCustomers


( Sizeof customer record )
( ---------------------- )

integer gnCust.recsize


: field>

  // Compile time:  ( n1 n2 --- n3 )
  //     <n1> is previous record size (total)
  //     <n2> is field size
  //     <n3> is updated record size
  build>
      over                                                     // ( n1 n2 --- n1 n2 n1 )
      1 allot                                                  // ( n1 n2 n1 --- n1 n2 n1 addr ) Alloc a cell to store field offst
      !                                                        // ( n1 n2 n1 addr --- n1 n2 )
      +                                                        // ( n1 n2 --- n3 )

  // Run time:     ( addr1 --- addr2 )
  //     <addr1> is base record address
  //     <addr2> is field address
  does>
      @                                                        // ( addr1 addr2 --- addr1 n ) Read field size
      +                                                        // ( addr1 n --- addr2 ) Compute field address

  ;  // field

0                                                              // Initialize custormer record sizeof with 0
20 1+       field> cust.name                                   // Name:     string, 20 chars
15 1+       field> cust.surname                                // Surname:  string, 15 chars
WordSize    field> cust.zip                                    // Zip:      numeric, integer
gnCust.recsize !                                               // Update customer record size

: customer>

  // Compile time:  ( --- )
  build>
      gnCust.recsize @ allot drop                              // Allocate space to store record

  // Run time:      ( --- addr )   ( Just return address )
  does>

  ;  // customer>


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : Frame                                                                                                          \\
//      Stack : ( --- )                                                                                                        \\
// Description : Install the initial on screen.                                                                                 \\
//    Callers :                                                                                                                \\
//      Calls :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Frame

  ?UpdateCursor @                                              // ( --- f ) Save flag
  ?UpdateCursor 0!                                             // Do not update cursor when drawing

  0  0 " Current record: #                       records stored            Time:         " !xystring
  0  1 " --------------------------------------------------------------------------------" !xystring

  0 23 " --------------------------------------------------------------------------------" !xystring
  0 24 gotoXY
  0x18 emit  0x19 emit
  " :fields PgUp/PgDn:records F2:new F3:modify F4:delete F5:search ESC:exit" !string

  ?UpdateCursor !                                              // ( f --- ) Restore Update cursor flag

  ;  // Frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : showRecordsNb                                                                                                  \\
//      Stack : ( --- )                                                                                                        \\
// Description : Show the nb of records into the file.                                                                          \\
//    Callers :                                                                                                                \\
//      Calls :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: showRecordsNb
  ghCustomers @ FileSize WordSize -  gnCust.recsize @  /       // Ask for the size, then compute records nb
  34 0 gotoXY  5 .R                                            // Show nb
  ;  // showRecordsNb


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : OpenBase                                                                                                       \\
//      Stack : ( --- )                                                                                                        \\
// Description : Open customers file.                                                                                           \\
//    Callers :                                                                                                                \\
//      Calls :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: OpenBase

  " CUSTOMRS.DAT" R/W OpenFile
  dup -1 == if
      cls " Unable to open file. Error #" !string              // Error msg
      FileIOStatus .
      Abort                                                    // Return to FROTH interpret
  endif

  ghCustomers !                                                // Save file handle

  // Initial creation of the file ?
  FileIOStatus 2 == if
      integer dummy  -1 dummy !
      ghCustomers @   dummy  WordSize  WriteFile               // The four 1st bytes are a pointer to the 1st deleted record
      drop                                                     // Don't test on write...
  endif

  showRecordsNb                                                // Show the number of records into file
  ;  // OpenBase


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : CloseBase                                                                                                      \\
//      Stack : ( --- )                                                                                                        \\
// Description : Close customers file.                                                                                          \\
//    Callers :                                                                                                                \\
//      Calls :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: CloseBase
  ghCustomers @ CloseFile
  ;  // CloseBase


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : Customers                                                                                                      \\
//      Stack : ( --- )                                                                                                        \\
// Description : The top word of the application.                                                                               \\
//    Callers :                                                                                                                \\
//       Call :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Customers
  C_NormalText ScreenBackground !                              // The screenbackground is Black
  ?UpdateCursor 1!                                             // Update cursor when drawing
  cls                                                          // Clear screen
  Frame                                                        // Install the initial frame
  OpenBase                                                     // Open customers file
  boolean bStop   bStop 0!                                     // Exit loop ?
  begin
      79 24 gotoXY key                                         // ( --- n )
      fkeys
      case
          KEY_ESC   of  bStop 1!  endof                        // ESC -> exit application
      endcase
      screen
      bStop @0!=
  until
  CloseBase                                                    // Close customers file
  ;  // Customers


// ---------------------------------------------------------------------------------------------------------------------------- \\
//       Word : g                                                                                                              \\
//      Stack : ( --- )                                                                                                        \\
// Description : Call the application, when under developement.                                                                 \\
//    Callers :                                                                                                                \\
//       Call :                                                                                                                \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: g
  3 SetVideoMode
  Customers
  84 SetVideoMode
  ;  // g


( Show informations about code and static data size )
( ------------------------------------------------- )
Infos

