// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: FRED.FTH       Author: Oliver SINGLA	 Date: Dec,16,95    Version: 0.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 NIM game.												        \\
//															        \\
// Release:														        \\
//     0.00    Dec,16,1995												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;


( -------------------------- )
( Some application constants )
( -------------------------- )

60000  constant BUFFER-SIZE				       // The buffers for all edited source
63     constant MAXLEN-FILENAME				       // Max length for a filename
10     constant MAXNB-BUFFERS				       // Max number of buffers
131    constant MAXLEN-LINE				       // Max length of an editable line


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : CreateAllBuffers											        \\
// 	 Stack : ( --- addr )  for run-time word									        \\
// Description : Create the space for all source buffers								        \\
//    Callers : buffers												        \\
// 	 Calls : BUFFER-SIZE												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  CreateAllBuffers
   build>
       BUFFER-SIZE ballot
       drop
   does>
   ;   // CreateAllBuffers

( Create the space for all buffers )
CreateAllBuffers buffers


( Length of a buffer directory )
:  BufferDir.filename	       ;
:  BufferDir.StartOfText       BufferDir.filename      MAXLEN-FILENAME +  ;
:  BufferDir.EndOfText	       BufferDir.StartOfText   WordSize+   ;
:  BufferDir.LengthText	       BufferDir.EndOfText     WordSize+   ;
:  BufferDir.AllocatedSize     BufferDir.LengthText    WordSize+   ;
:  BufferDir.unloaded	       BufferDir.AllocatedSize WordSize+   ;
0 BufferDir.unloaded 1+ constant   BufferDir-size

:  CreateBuffersDir
   build>
       MAXNB-BUFFERS BufferDir-size * ballot		       // ( --- addr )
       MAXNB-BUFFERS BufferDir-size * 0 fill		       // Zero all the dir buffer
   does>
       swap BufferDir-size * +				       // ( n addr1 --- addr2 )
   ;   // CreateBuffersDir

CreateBuffersDir   BuffersDir


( ---------------- )
( Global variables )
( ---------------- )

int    CurrentBuffer#					       // Current buffer # in use


( ---------------------------- )
( Functions help to debug FRED )
( ---------------------------- )

" Fred-dbg" "load


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : NewLine->Buffer											        \\
// 	 Stack : ( bufaddr addrz --- )											        \\
// Description : Bring a *new* line at the end of the buffer								        \\
//    Callers : ReadFile->Buffer											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  NewLine->Buffer
   const StrAddr					       // Line address
   const BuffAddr					       // Buffer address
StrAddr type StrAddr $mlen . quit
cr BuffAddr h. space StrAddr h. space '[ emit StrAddr type '] emit 4 spaces

   ( Add the string at the end of the buffer )
   BuffAddr BufferDir.EndOfText				       // ( --- addr1 )			      Destination address
   StrAddr						       // ( addr1 --- addr1 addrz2 )	      Source address
   dup $len						       // ( addr1 addrz2 --- addr1 addrz2 n ) Nb of bytes to move
   cmove

   ( Update buffer directory )

   ;   // NewLine->Buffer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ReadFile->Buffer											        \\
// 	 Stack : ( fh buffer# --- )											        \\
// Description : Read an opened-file in a buffer									        \\
//    Callers : File->Buffer												        \\
// 	 Calls : MAXLEN-LINE BuffersDir NewLine->Buffer									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ReadFile->Buffer
   const buffer#					       // ( fh buffer# --- fh )	   Save the buffer #
   const fh						       // ( fh --- )		   Save the file handle
   MAXLEN-LINE fstring line$				       // Buffer to hold each line read
   begin
       fh line$ line$ $mlen read-line drop		       // Read next line from source file
       fh ?eof not
   while
       buffer# BuffersDir line$ NewLine->Buffer		       // Bring this new line to buffer
   repeat
   ;   // ReadFile->Buffer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : File->Buffer												        \\
// 	 Stack : ( addrz n --- )											        \\
// Description : Bring the given file (if exist) to the buffer #n							        \\
//    Callers : fred													        \\
// 	 Calls : ReadFile->Buffer											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  File->Buffer
   const buffer#					       // ( addrz n --- addrz )	   Save the buffer #
   cstring filename$					       // ( addrz --- )		   Save the filename
   filename$ R/W open-file				       // ( addrz n --- fh )
   dup -1 == if
       200 fstring err$
       " Unable to open/create file " err$ $!
       filename$ err$ $+
       "  - fileiostatus = " err$ $+
       fileiostatus <#S> err$ $+
       err$ "error
   endif
   const fh						       // File handle
   fh buffer# ReadFile->Buffer				       // ( fh n --- )
   fh close-file					       // Close the opened file
   ;   // File->Buffer


:  FirstInitialize
   CurrentBuffer# 0!					       // 1st buffer in use
   MAXNB-BUFFERS 0 do					       // Clear each buffer directory
       I BuffersDir  0  BufferDir-size  fill
   enddo
   ;   // FirstInitialize


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : fred													        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Call the full screen editor, 1st buffer is the given file						        \\
//    Callers : g													        \\
// 	 Calls : CurrentBuffer# File->Buffer FirstInitialize								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  fred
   ?string						       // Is given a valid string ?
   FirstInitialize
   CurrentBuffer# @  File->Buffer			       // Bring file to buffer # 0
   ;   // fred

:  g   " toto.$$$" fred ;


( Print informations about Code/Data size )
( --------------------------------------- )

infos

