// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: GTEST.FTH      Author: Oliver SINGLA	 Date: Nov,1,94	    Version: 2.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:														        \\
//     Graphics test.													        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
: <task> ;

( Colors used )
screen
black LightGray BackForeGround const C_NormalText	       // Normal text

( Some constants )
50000 const NB_PIXELS					       // Nb of random pixels
int gnHeightTittle					       // Height of tittle zone


( Make GRAPHICS the context vocabulary )
graphics


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : point>													        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  point>
   build>						       // Compile-time: ( n --- )
       2* 2* ballot drop				       // Room for <n> couples of word
   does>						       //     Run-time: ( n --- addr )	 <addr> is address of x-point
       swap 2* 2* +					       // ( n addr1 --- addr2 )
   ;   // point>


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputeHeightTittle											        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers : DrawSomePixels DrawSomeLines DrawSomeBoxes								        \\
// 	 Calls : gnHeightTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputeHeightTittle
   #grows screen #rows graphics /
   gnHeightTittle !
   ;   // ComputeHeightTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PressKey												        \\
// 	 Stack : ( --- )												        \\
// Description : Type a msg, then wait until a key is pressed.								        \\
//    Callers : DrawSomePixels DrawSomeLines DrawSomeBoxes								        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PressKey
   screen
   #cols 11 - 0 " Press a key" !xystring
   key drop
   #cols 11 - 0 gotoxy clreol
   graphics
   ;   // PressKey


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawSomePixels												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw some pixels.											        \\
//    Callers : PerformTest_pixels:8											        \\
// 	 Calls : NB_PIXELS:3 gnHeightTittle ComputeHeightTittle PressKey						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawSomePixels

   ComputeHeightTittle
   screen
   0 0 gotoxy
   !string
   " : SetPixel / GetPixel" !string
   graphics

   NB_PIXELS point> points				       // All points pixel-ed

   ( Draw some pixels, remember each pixel )
   0  NB_PIXELS 1-  make
       0  #gcols 1-  rnd				       // ( --- x )
       gnHeightTittle @	 #grows 1-  rnd			       // ( x --- x y )
       ddup						       // ( x y --- x y x y )
       0  #gcolors 1-  rnd				       // ( x y x y --- x y x y color )
       SetPixel						       // ( x y x y color --- x y )
       I points						       // ( x y --- x y addr )
       rot over W!					       // ( x y addr --- y addr )
       2+ W!						       // ( y addr --- )
   endmake

   ( Restore each pixel with its previous color )
   0  NB_PIXELS 1-  make
       I points						       // ( --- addr )
       dup W@ swap					       // ( addr --- x addr )
       2+ W@						       // ( x addr --- x y )
       0
       SetPixel						       // ( x y color --- )
   endmake

   PressKey						       // Pause

   ;   // DrawSomePixels


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_pixels											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test pixels definitions.										        \\
//    Callers : PerformTest												        \\
// 	 Calls : DrawSomePixels:8											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PerformTest_pixels
   case
       'A of  " 320x200x16"  DrawSomePixels  endof
       'B of  " 640x200x16"  DrawSomePixels  endof
       'C of  " 640x350x4"   DrawSomePixels  endof
       'D of  " 640x350x16"  DrawSomePixels  endof
       'E of  " 640x480x2"   DrawSomePixels  endof
       'F of  " 640x480x16"  DrawSomePixels  endof
       'G of  " 320x200x256" DrawSomePixels  endof
       'H of  " 800x600x16"  DrawSomePixels  endof
   endcase
   ;   // PerformTest_pixels


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawSomeLines												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw some lines.											        \\
//    Callers : PerformTest_lines:8											        \\
// 	 Calls : gnHeightTittle:3 ComputeHeightTittle PressKey								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawSomeLines

   ComputeHeightTittle
   screen
   0 0 gotoxy
   !string
   " : lines" !string
   graphics

   int nColor1
   int nColor2
   10 times

       0 #gcolors 1- rnd  nColor1 !
       gnHeightTittle @	 #grows 1-  make
	   0  gnHeightTittle @  #gcols 1-  I  nColor1 @	 line
       endmake

       begin
	   0 #gcolors 1- rnd  nColor2 !
	   nColor2 nColor1 @!=
       until
       #gcols 1-  0  -1	 +make
	   0  gnHeightTittle @  I  #grows 1-  nColor2 @	 line
       endmake

   endtimes

   PressKey						       // Pause

   ;   // DrawSomeLines


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_lines											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test lines definitions.										        \\
//    Callers : PerformTest												        \\
// 	 Calls : DrawSomeLines:8											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PerformTest_lines
   case
       'A of  " 320x200x16"  DrawSomeLines  endof
       'B of  " 640x200x16"  DrawSomeLines  endof
       'C of  " 640x350x4"   DrawSomeLines  endof
       'D of  " 640x350x16"  DrawSomeLines  endof
       'E of  " 640x480x2"   DrawSomeLines  endof
       'F of  " 640x480x16"  DrawSomeLines  endof
       'G of  " 320x200x256" DrawSomeLines  endof
       'H of  " 800x600x16"  DrawSomeLines  endof
   endcase
   ;   // PerformTest_lines


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawSomeBoxes												        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers : PerformTest_boxes:8											        \\
// 	 Calls : gnHeightTittle:2 ComputeHeightTittle PressKey								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawSomeBoxes

   ComputeHeightTittle
   screen
   0 0 gotoxy
   !string
   " : boxes" !string
   graphics

   750 times
       0  #gcols 1-  rnd
       gnHeightTittle @	 #grows 1-  rnd
       0  #gcols 1-  rnd
       gnHeightTittle @	 #grows 1-  rnd
       0  #gcolors 1-  rnd
       FILLED
       box
   endtimes

   PressKey						       // Pause

   ;  // DrawSomeBoxes


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_boxes											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test boxes definitions.										        \\
//    Callers : PerformTest												        \\
// 	 Calls : DrawSomeBoxes:8											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:   PerformTest_boxes
    case
        'A of  " 320x200x16"  DrawSomeBoxes  endof
        'B of  " 640x200x16"  DrawSomeBoxes  endof
        'C of  " 640x350x4"   DrawSomeBoxes  endof
        'D of  " 640x350x16"  DrawSomeBoxes  endof
        'E of  " 640x480x2"   DrawSomeBoxes  endof
        'F of  " 640x480x16"  DrawSomeBoxes  endof
        'G of  " 320x200x256" DrawSomeBoxes  endof
        'H of  " 800x600x16"  DrawSomeBoxes  endof
    endcase
    ;  // PerformTest_boxes


(
31 12 sprite s_Alien_0
    '. screen black	   graphics s_Alien_0 >color_sprite
    'w screen white	   graphics s_Alien_0 >color_sprite
    'y screen yellow	   graphics s_Alien_0 >color_sprite
    'c screen cyan	   graphics s_Alien_0 >color_sprite
    'r screen red	   graphics s_Alien_0 >color_sprite
    'm screen magenta	   graphics s_Alien_0 >color_sprite
    0  dup " .www.......................www." s_Alien_0 >sprite
    1+ dup " .wmmw.....................wmmw." s_Alien_0 >sprite
    1+ dup " .wmmmww.................wwmmmw." s_Alien_0 >sprite
    1+ dup " .wwwmmmw...............wmmmww.." s_Alien_0 >sprite
    1+ dup " w...wmmmw.....yyy.....wmmmw...w" s_Alien_0 >sprite
    1+ dup " .....wwmmww..ccccc..wwmmww....." s_Alien_0 >sprite
    1+ dup " ....w..wmmmwccrccrcwmmmw..w...." s_Alien_0 >sprite
    1+ dup " ........wmmwcrrcrrcwmmw........" s_Alien_0 >sprite
    1+ dup " ........wwwwcccccccwwww........" s_Alien_0 >sprite
    1+ dup " .......w....ccccccc....w......." s_Alien_0 >sprite
    1+ dup " ............ccrrrcc............" s_Alien_0 >sprite
    1+	   " .............crcrc............." s_Alien_0 >sprite


33 11 sprite s_Alien_1
    '. screen black	   graphics s_Alien_1 >color_sprite
    'w screen white	   graphics s_Alien_1 >color_sprite
    'y screen yellow	   graphics s_Alien_1 >color_sprite
    'c screen cyan	   graphics s_Alien_1 >color_sprite
    'r screen red	   graphics s_Alien_1 >color_sprite
    'm screen magenta	   graphics s_Alien_1 >color_sprite
    0  dup " ..w...........................w.." s_Alien_1 >sprite
    1+ dup " .wmww.......................wwmw." s_Alien_1 >sprite
    1+ dup " .wmmmww...................wwmmmw." s_Alien_1 >sprite
    1+ dup " .wmmmmmww......yyy......wwmmmmmw." s_Alien_1 >sprite
    1+ dup " w.wmmmmmmww...ccccc...wwmmmmmmw.w" s_Alien_1 >sprite
    1+ dup " ...wwwmmmmmw.cr.cr.c.wmmmmmwww..." s_Alien_1 >sprite
    1+ dup " ......wwmmmmwcrrcrrcwmmmmww......" s_Alien_1 >sprite
    1+ dup " .....w..wwwmwcccccccw.www..w....." s_Alien_1 >sprite
    1+ dup " ...........w.ccccccc.w..........." s_Alien_1 >sprite
    1+ dup " ..........w..ccrrrcc..w.........." s_Alien_1 >sprite
    1+	   " ..............crcrc.............." s_Alien_1 >sprite


33 8 sprite s_Alien_2
    '. screen black	   graphics s_Alien_2 >color_sprite
    'w screen white	   graphics s_Alien_2 >color_sprite
    'y screen yellow	   graphics s_Alien_2 >color_sprite
    'c screen cyan	   graphics s_Alien_2 >color_sprite
    'r screen red	   graphics s_Alien_2 >color_sprite
    'm screen magenta	   graphics s_Alien_2 >color_sprite
    0  dup " .www...........yyy...........www." s_Alien_2 >sprite
    1+ dup " wmmmwwwwww....ccccc....wwwwwwmmmw" s_Alien_2 >sprite
    1+ dup " wmmmmmmmmmww.crrcrrc.wwmmmmmmmmmw" s_Alien_2 >sprite
    1+ dup " .wwwmmmmmmmmwcr.cr.cwmmmmmmmmwww." s_Alien_2 >sprite
    1+ dup " w...wwwwwwmmwcccccccwmmwwwwww...w" s_Alien_2 >sprite
    1+ dup " ....w.....ww.ccccccc.ww.....w...." s_Alien_2 >sprite
    1+ dup " .........w...ccrrrcc...w........." s_Alien_2 >sprite
    1+	   " ..............crcrc.............." s_Alien_2 >sprite


33 8 sprite s_Alien_3
    '. screen black	   graphics s_Alien_3 >color_sprite
    'w screen white	   graphics s_Alien_3 >color_sprite
    'y screen yellow	   graphics s_Alien_3 >color_sprite
    'c screen cyan	   graphics s_Alien_3 >color_sprite
    'r screen red	   graphics s_Alien_3 >color_sprite
    'm screen magenta	   graphics s_Alien_3 >color_sprite
    0  dup " ...............yyy..............." s_Alien_3 >sprite
    1+ dup " ..............ccccc.............." s_Alien_3 >sprite
    1+ dup " .wwwwwwwwwww.crrcrrc.wwwwwwwwwww." s_Alien_3 >sprite
    1+ dup " wmmmmmmmmmmmwc.rc.rcwmmmmmmmmmmmw" s_Alien_3 >sprite
    1+ dup " wmmmmmmmmmmmwcccccccwmmmmmmmmmmmw" s_Alien_3 >sprite
    1+ dup " .wwwwwwwwwww.ccccccc.wwwwwwwwwww." s_Alien_3 >sprite
    1+ dup " ..w...w...w..ccrrrcc..w...w...w.." s_Alien_3 >sprite
    1+	   " ..w...w...w...crcrc...w...w...w.." s_Alien_3 >sprite


33 8 sprite s_Alien_4
    '. screen black	   graphics s_Alien_4 >color_sprite
    'w screen white	   graphics s_Alien_4 >color_sprite
    'y screen yellow	   graphics s_Alien_4 >color_sprite
    'c screen cyan	   graphics s_Alien_4 >color_sprite
    'r screen red	   graphics s_Alien_4 >color_sprite
    'm screen magenta	   graphics s_Alien_4 >color_sprite
    0  dup " ...............yyy..............." s_Alien_4 >sprite
    1+ dup " ..............ccccc.............." s_Alien_4 >sprite
    1+ dup " .........www.c.rc.rc.www........." s_Alien_4 >sprite
    1+ dup " ...wwwwwwmmmwcrrcrrcwmmmwwwwww..." s_Alien_4 >sprite
    1+ dup " .wwmmmmmmmmmwcccccccwmmmmmmmmmww." s_Alien_4 >sprite
    1+ dup " wmmmmmmmmwww.ccccccc.wwwmmmmmmmmw" s_Alien_4 >sprite
    1+ dup " wmmwwwwww....ccrrrcc....wwwwwwmmw" s_Alien_4 >sprite
    1+	   " .ww...........crcrc...........ww." s_Alien_4 >sprite


33 11 sprite s_Alien_5
    '. screen black	   graphics s_Alien_5 >color_sprite
    'w screen white	   graphics s_Alien_5 >color_sprite
    'y screen yellow	   graphics s_Alien_5 >color_sprite
    'c screen cyan	   graphics s_Alien_5 >color_sprite
    'r screen red	   graphics s_Alien_5 >color_sprite
    'm screen magenta	   graphics s_Alien_5 >color_sprite
    0  dup " ...............yyy..............." s_Alien_5 >sprite
    1+ dup " ..........w...ccccc...w.........." s_Alien_5 >sprite
    1+ dup " ...........w.cr.cr.c.w..........." s_Alien_5 >sprite
    1+ dup " .....w..wwwmwcrrcrrcwmwww..w....." s_Alien_5 >sprite
    1+ dup " ......wwmmmmwcccccccwmmmmww......" s_Alien_5 >sprite
    1+ dup " ...wwwmmmmmw.ccccccc.wmmmmmwww..." s_Alien_5 >sprite
    1+ dup " w.wmmmmmmww..ccrrrcc..wwmmmmmmw.w" s_Alien_5 >sprite
    1+ dup " .wmmmmmww.....crcrc.....wwmmmmmw." s_Alien_5 >sprite
    1+ dup " .wmmmww...................wwmmmw." s_Alien_5 >sprite
    1+ dup " .wmww.......................wwmw." s_Alien_5 >sprite
    1+	   " ..w...........................w.." s_Alien_5 >sprite


31 11 sprite s_Alien_6
    '. screen black	   graphics s_Alien_6 >color_sprite
    'w screen white	   graphics s_Alien_6 >color_sprite
    'y screen yellow	   graphics s_Alien_6 >color_sprite
    'c screen cyan	   graphics s_Alien_6 >color_sprite
    'r screen red	   graphics s_Alien_6 >color_sprite
    'm screen magenta	   graphics s_Alien_6 >color_sprite
    0  dup " ..............yyy.............." s_Alien_6 >sprite
    1+ dup " .............ccccc.....w......." s_Alien_6 >sprite
    1+ dup " ........wwwwcrrcrrcwwww........" s_Alien_6 >sprite
    1+ dup " ........wmmwcr.cr.cwmmw........" s_Alien_6 >sprite
    1+ dup " ....w..wmmmwcccccccwmmmw..w...." s_Alien_6 >sprite
    1+ dup " .....wwmmww.ccccccc.wwmmww....." s_Alien_6 >sprite
    1+ dup " w...wmmmw...ccrrrcc...wmmmw...w" s_Alien_6 >sprite
    1+ dup " .wwwmmmw.....crcrc.....wmmmwww." s_Alien_6 >sprite
    1+ dup " .wmmmww.................wwmmmw." s_Alien_6 >sprite
    1+ dup " .wmmw.....................wmmw." s_Alien_6 >sprite
    1+	   " .www.......................www." s_Alien_6 >sprite


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : des_ships>												        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  des_ships>
   build>
       1 allot  !
   does>
       swap WordSize* 4 * +
   ;   // des_ships


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ,													        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ,
   1 AllotMore
   !
   ;   // ,


3 des_ships> alien_ship
    31 ,  12 ,	 0 ,  0 ,  s_alien_0 ,
    33 ,  11 ,  -1 ,  1 ,  s_alien_1 ,
    33 ,   8 ,	 0 ,  3 ,  s_alien_2 ,
    33 ,   8 ,	 0 ,  0 ,  s_alien_3 ,
    33 ,   8 ,	 0 ,  0 ,  s_alien_4 ,
    33 ,  11 ,	 0 ,  0 ,  s_alien_5 ,
    31 ,  11 ,	 1 ,  0 ,  s_alien_6 ,
)


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_sprites											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test sprite definitions.										        \\
//    Callers : PerformTest												        \\
// 	 Calls : DrawSomeBoxes:8											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PerformTest_sprites
   ;   // PerformTest_sprites


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : doTittle												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : DrawTittle												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  doTittle
   screen
   dup $len const len
   #cols  len 4 + -  2/	 const  x
   x 0 ' !xychar  x 1+ 0 len 2+ ' !xynchar  x len + 3 + 0 ' !xychar
   x 1 "   " !xystring  x 2+ 1 rot !xystring  x len + 2+ 1 "  " !xystring
   x 2 ' !xychar  x 1+ 2 len 2+ ' !xynchar  x len + 3 + 2 ' !xychar
   ;   // doTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTittle												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle on screen										        \\
//    Callers : Menu													        \\
// 	 Calls : doTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawTittle
   80 fstring tittle$
   " FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " GRAPHICS EXAMPLES AND TESTS" +$ doTittle
   ;   // DrawTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTestsMenu												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw the tests menu.											        \\
//    Callers : Menu													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawTestsMenu
   const y
   const x
   '~ >xylstring_char
   0 black yellow    >xylstring_attr
   1 black lightgray >xylstring_attr
   x y	    " ~{1}Ŀ" !xylstring
   x y  1 + " ~{1}       TESTS       " !xylstring
   x y  2 + " ~{1}Ĵ" !xylstring
   x y  3 + " ~{1}   ~{0}1~{1}. Pixels       " !xylstring
   x y  4 + " ~{1}   ~{0}2~{1}. Lines        " !xylstring
   x y  5 + " ~{1}   ~{0}3~{1}. Boxes        " !xylstring
   x y  6 + " ~{1}   ~{0}4~{1}. Sprites      " !xylstring
   x y  7 + " ~{1}Ĵ" !xylstring
   x y  8 + " ~{1} ~{0}ESC~{1}. Exit         " !xylstring
   x y  9 + " ~{1}" !xylstring
   ;   // DrawTestsMenu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawResolutionsMenu											        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw the resolutions menu.										        \\
//    Callers : Menu													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawResolutionsMenu
   const y
   const x
   '~ >xylstring_char
   0 black yellow    >xylstring_attr
   1 black lightgray >xylstring_attr
   x y  0 + " ~{1}Ŀ" !xylstring
   x y  1 + " ~{1}     RESOLUTIONS     " !xylstring
   x y  2 + " ~{1}Ĵ" !xylstring
   x y  3 + " ~{1} ~{0}A~{1}. 320 x 200 x 16   " !xylstring
   x y  4 + " ~{1} ~{0}B~{1}. 640 x 200 x 16   " !xylstring
   x y  5 + " ~{1} ~{0}C~{1}. 640 x 350 x 4    " !xylstring
   x y  6 + " ~{1} ~{0}D~{1}. 640 x 350 x 16   " !xylstring
   x y  7 + " ~{1} ~{0}E~{1}. 640 x 480 x 2    " !xylstring
   x y  8 + " ~{1} ~{0}F~{1}. 640 x 480 x 16   " !xylstring
   x y  9 + " ~{1} ~{0}G~{1}. 320 x 200 x 256  " !xylstring
   x y 10 + " ~{1} ~{0}H~{1}. 800 x 600 x 16   " !xylstring
   x y 11 + " ~{1}" !xylstring
   ;   // DrawResolutionsMenu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : FixResolution												        \\
// 	 Stack : ( c --- )												        \\
// Description : Fix the resolution, as given in DrawResolutionsMenu							        \\
//    Callers : InputAnswer:2												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  FixResolution
   graphics
   case
       'A of  320x200x16  endof
       'B of  640x200x16  endof
       'C of  640x350x4	  endof
       'D of  640x350x16  endof
       'E of  640x480x2	  endof
       'F of  640x480x16  endof
       'G of  320x200x256 endof
       'H of  0x58 800x600x16  endof
   endcase
   screen
   ;   // FixResolution


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest												        \\
// 	 Stack : ( c1 c2 --- )	 <c1> is graphics mode	  <c2> is operation						        \\
// Description : Perform the asked test (as given in DrawTestsMenu).							        \\
//    Callers : InputAnswer:2												        \\
// 	 Calls : PerformTest_pixels PerformTest_lines PerformTest_boxes							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PerformTest
   case
       '1 of  PerformTest_Pixels      endof
       '2 of  PerformTest_Lines	      endof
       '3 of  PerformTest_boxes	      endof
       '4 of  PerformTest_sprites     endof
   endcase
   ;   // PerformTest


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InputAnswer												        \\
// 	 Stack : ( x y --- tf )	  if exit application (ESC)								        \\
// 		 ( x y --- n ff ) if an operation is asked.								        \\
// Description : Draw input box, then wait for answer.									        \\
//    Callers : Menu													        \\
// 	 Calls : FixResolution:2 PerformTest:2										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InputAnswer

  const y
  const x
  '~ >xylstring_char
  0 black yellow    >xylstring_attr
  1 black lightgray >xylstring_attr
  x y	  " ~{1}Ŀ" !xylstring
  x y 1 + " ~{1} Enter your choice (example: ~{0}A1~{1} or ~{0}1A~{1}) ?    " !xylstring
  x y 2 + " ~{1}" !xylstring

  x 42 + const x
  y 1+	 const y
  FALSE ilogical lExit?					       // Exit loop ?
  FALSE ilogical lEsc?					       // Exit application ?
  0 iint nIndex						       // Index of key to input
  begin

      nIndex @ case

	  0 of
	      x y ddup 0x20 !xychar  gotoxy  key charuppercase  const cKey0
	      cKey0 0x1B == if
		  lEsc? true!
	      else
		  cKey0 '1 '4 >=<=  cKey0 'A 'H >=<=  or  if
		      cKey0 emit
		      nIndex 1+!
		  endif
	      endif
	  endof

	  1 of
	      x 1+ y ddup 0x20 !xychar  gotoxy  key charuppercase  const cKey1
	      cKey1 0x1B == if
		  lEsc? true!
	      else
		  cKey1 0x08 == if
		      x y 0x20 !xychar  nIndex 1-!
		  else
		      ckey1  cKey0 '1 '4 >=<= if
			  'A 'H
		      else
			 '1 '4
		      endif
		      >=<=  if
			  cKey1 emit
			  nIndex 1+!
		      endif
		  endif
	      endif
	  endof

	  2 of
	      x 2 + y ddup 0x20 !xychar	 gotoxy	 key
	      case
		  0x1B of  lEsc? true!			    endof
		  0x0D of  lExit? true!			    endof
		  0x08 of  x 1+ y 0x20 !xychar  nIndex 1-!  endof
	      endcase
	  endof

      endcase

      lExit? true?  lEsc? true?	 or
  until

  lEsc? true? if
      TRUE
  else
      cKey0 '1 '4 >=<= if
	  cKey1 FixResolution
	  cKey1 cKey0 PerformTest
      else
	  cKey0 FixResolution
	  cKey0 cKey1 PerformTest
      endif
      FALSE
  endif

  ;  // InputAnswer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Menu													        \\
// 	 Stack : ( --- f )    <f> : exit application ?									        \\
// Description : Draw menus, wait then perform asked operation, then exit.						        \\
//    Callers : gtest													        \\
// 	 Calls : DrawTittle DrawTestsMenu DrawResolutionsMenu InputAnswer						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Menu
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   DrawTittle						       // Draw tittle application.
   12 4 DrawResolutionsMenu				       // Draw the menu of available resolutions
   42 4 DrawTestsMenu					       // Draw the menu of available tests
   17 #rows 3 - InputAnswer				       // ( --- n )
   if
       TRUE						       // Exit application
   else
       FALSE						       // Do not exit application
   endif
   ;   // Menu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : gtest													        \\
// 	 Stack : ( --- )												        \\
// Description : Start the graphics definitions test.									        \\
//    Callers : g													        \\
// 	 Calls : C_NormalText Menu											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  gtest
   GetVideoMode						       // ( --- n )
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   C_NormalText ScreenBackground !			       // The screenbackground is Black
   cls							       // Clear screen
   ?UpdateCursor true!					       // Update cursor when printing on screen
   begin
       menu						       // Perform asked operation
   until
   drop //  SetVideoMode				       // ( n --- )
  ;  // gtest

: g gtest 0 15 gotoxy ;


( Print informations about Code/Data size )
( --------------------------------------- )

infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

\ " \\froth\\examples\\gtest.exe" save.exe
\ cr " GTEST.EXE app size is " type . "  bytes" type

