// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: Life.fth       Author: Oliver SINGLA	 Date: Dec, 27, 94  Version: 2.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 CONWAY's life game.											        \\
//															        \\
// Release:														        \\
//     2.00    Jan-03-96												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

forget <task>
: <task> ;

( Version # )
20000 const #version					       // v 2.00.a

( Make SCREEN the context vocabulary )
screen

( Colors used )
black LightGray BackForeGround const C_NormalText	       // Normal text
DarkGray const CellOffColor				       // Color of no cell (graphics mode)
   white const CellOnColor				       // Color of cell	   (graphics mode)

( Constants used )
640 const XSIZE						       // Max width
464 const YSIZE						       // Max height
 16 const XINC						       // X increment inside cells editor
  8 const YINC						       // Y increment inside cells editor
   0 const EVEN						       // Even generation (2, 4, ... )
   1 const ODD						       // Odd  generation (1, 3, ... )
'. const char_cell_off					       // Character used as cell off inside editor
'o const char_cell_on					       // Character used as cell on  inside editor
0x07 const char_cell_off_curr				       // Character used as cell off inside editor
0x0a const char_cell_on_curr				       // Character used as cell on  inside editor

( Keys used )
FKeys
KEY_F1  const KEY_HELP					       // F1  : Help
KEY_F2  const KEY_NEXT_GEN				       // F2  : Compute and show next generation
KEY_F3  const KEY_AUTO					       // F3  : Automatic mode (flip flop)
KEY_F10 const KEY_EDIT					       // F4  : Allows to edit the cells
KEY_ESC const KEY_BYE					       // ESC : Bye application
0x20    const KEY_CHGCELL				       // Change the state of the cell under the cursor (CellsEditor)
Screen

( Global variables )
int generation#						       // Generation (1, 2, ... )
int gnPopulationsNb					       // Nb of living cells
int gnEditor_X						       // Current column within editor
int gnEditor_Y						       // Current row within editor
int gnCornerTopLeft_X					       // Column of the top/left corner
int gnCornerTopLeft_Y					       // Row of the top/left corner
logical EditorActive?					       // Is editor active ?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InfosLineNo												        \\
// 	 Stack : ( --- n )												        \\
// Description : Return the 1st row of informations.									        \\
//    Callers : ShowGenPop ShowAvailableKeysCmd Frame ShowGeneration NextGeneration:2 AutomaticMode Help:2 CellsEditor:3       \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InfosLineNo
   #rows 1-
   ;   // InfosLineNo


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : CellsWorld												        \\
// 	 Stack : ( --- )												        \\
// Description : Creating word for Cells										        \\
//    Callers : cells													        \\
// 	 Calls : XSIZE YSIZE												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  CellsWorld
   build>
       XSIZE YSIZE *  ballot				       // Allocate space for each cells
       drop						       // ( addr --- )
   does>						       // ( x y addr1 --- addr2 )
//     swap XSIZE * + +
       ( mov   eax, [esi + 4] )	 [: 8B 46 04 :]
       ( imul  eax, XSIZE     )	 [: 69 C0 80 02 00 00 :]
       ( add   eax, [esi]     )	 [: 03 06 :]
       ( add   esi, 8	      )	 [: 83 C6 08 :]
       ( add   [esi], eax     )	 [: 01 06 :]
   ;   // CellsWorld


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Cells													        \\
// 	 Stack : RunTime: ( x y -- addr )										        \\
// Description : Return address for the cell (x, y)    (x is the width, y is the height)				        \\
//    Callers : PutCell GetCell RvsCell InitThings ZeroLife:5 ComputeNextGeneration ShowGeneration Editor_ShowGeneration       \\
// 		 Editor_ShowCells											        \\
// 	 Calls : CellsWorld												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
dynamic CellsWorld cells


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PutCell												        \\
// 	 Stack : ( n x y --- )												        \\
// Description : Store a living cell (1) or dead cell (0), at column <x>, row <y>.					        \\
//    Callers :													        \\
// 	 Calls : generation# cells											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PutCell
   Cells						       // ( n x y --- n addr )
   swap							       // ( n addr --- addr n )
   generation# @ 2 mod					       // ( addr n --- addr n 0|1 )
   swap							       // ( addr n 0|1 --- addr 0|1 n )
   if
       swap SetBit					       // ( n addr --- )
   else
       swap ResetBit					       // ( n addr --- )
   endif
   ;   // PutCell


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : GetCell												        \\
// 	 Stack : ( x y --- n )												        \\
// Description : Return 0 (dead cell) or 1 (living cell), from column <x>, row <y>.					        \\
//    Callers : changes> Editor_ChangeCell:2 CellsEditor:2								        \\
// 	 Calls : generation# cells											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  GetCell
   Cells						       // ( x y --- addr )
   generation# @ 2 mod					       // ( addr --- addr 0|1 )
   swap							       // ( addr 0|1 --- 0|1 addr )
   GetBit						       // ( n1 addr --- n2 )
   ;   // GetCell


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : RvsCell												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Reverse the Dead/Living status of the cell, at column <x>, row <y>.					        \\
//    Callers : Editor_ChangeCell											        \\
// 	 Calls : generation# cells											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  RvsCell
   Cells						       // ( x y --- addr )
   generation# @ 2 mod					       // ( addr --- addr 0|1 )
   swap							       // ( addr 0|1 --- 0|1 addr )
   ComplBit						       // ( n addr --- )
   ;   // RvsCell


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : changes>												        \\
// 	 Stack : CompileTime: ( --- )											        \\
// 		     RunTime: ( n --- x y f )										        \\
// Description : Create the array of which changed after a computation.							        \\
//    Callers : changes												        \\
// 	 Calls : XSIZE:2 YSIZE GetCell											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  changes>
   build>
       XSIZE YSIZE * WordSize*				       // Room for: XY (dword)
       ballot drop					       // Ask for room
   does>
       over -1 == if
	   under
       else
	   swap WordSize*  +				       // ( n addr1 --- addr2 )
	   @ XSIZE /mod					       // ( addr --- x y )
	   ddup getcell					       // ( x y --- x y f )
       endif
   ;   // changes>


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : changes												        \\
// 	 Stack : ( n --- x y f )											        \\
// Description : Ask the array of which changed (updated by ComputeGeneration)						        \\
//    Callers : InitThings ComputeNextGeneration ShowGeneration life							        \\
// 	 Calls : changes>												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
dynamic changes> changes


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitThings												        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize the game.											        \\
//    Callers : life													        \\
// 	 Calls : XSIZE YSIZE generation# gnPopulationsNb gnEditor_X gnEditor_Y gnCornerTopLeft_X:2 gnCornerTopLeft_Y:2	        \\
// 		 EditorActive? cells changes										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InitThings

   ( Allocate the huge array to store each cell state )
   alloc cells
   alloc changes

   ( Set initial editing position, the center of the array )
   XSIZE #cols - 2/  gnCornerTopLeft_X !
   YSIZE #rows - 2/  gnCornerTopLeft_Y !
   #cols 2/  gnCornerTopLeft_X @  +  gnEditor_X !
   #rows 2/  gnCornerTopLeft_Y @  +  gnEditor_Y !

   ( Initial population is empty )
   gnPopulationsNb 0!

   ( 1st generation )
   generation# 1!

   ( Is editor active ? )
   EditorActive? true!

   ;   // InitThings


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowGenPop												        \\
// 	 Stack : ( --- )												        \\
// Description : Print the generation # and the nb of populations.							        \\
//    Callers : Frame NextGeneration Editor_ChangeCell CellsEditor							        \\
// 	 Calls : generation# gnPopulationsNb InfosLineNo								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowGenPop
   InfosLineNo
    4 over gotoXY   6  gnPopulationsNb @  .R
   14 swap gotoXY   4  generation# @	  .R
   ;   // ShowGenPop


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowAvailableKeysCmd											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the available keys.										        \\
//    Callers : Frame AutomaticMode CellsEditor									        \\
// 	 Calls : EditorActive? InfosLineNo										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowAvailableKeysCmd
   '~ >xylstring_char
   0 black white       >xylstring_attr
   1 black lightgray   >xylstring_attr
   InfosLineNo 19 swap					       // ( --- x y )
   EditorActive? true? if
	   " ~{0}F1~{1} help ~{0}SPACE~{1} on/off [^]~{0}\18\19\1b\1a~{1} mov ~{0}F10~{1} ExitEd ~{0}ESC~{1} bye"
   else
	   " ~{0}F1~{1} help ~{0}F2~{1} next generation ~{0}F3~{1} automatic ~{0}F10~{1} editor ~{0}ESC~{1} bye"
   endif
   !xylstring
   clreol						       // Clear until end of line
   ;  // ShowAvailableKeysCmd


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Frame													        \\
// 	 Stack : ( --- )												        \\
// Description : Print the frame of the game.										        \\
//    Callers : Help CellsEditor life											        \\
// 	 Calls : InfosLineNo ShowGenPop ShowAvailableKeysCmd								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Frame
   InfosLineNo
    1 over " Pop " !xystring
   11 swap " Gen " !xystring
   ShowAvailableKeysCmd					       // Draw the availble keys.
   ShowGenPop						       // Print the generation # and the nb of populations.
   ;  // Frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ZeroLife												        \\
// 	 Stack : ( --- )												        \\
// Description : Empty all the cells.											        \\
//    Callers : life													        \\
// 	 Calls : XSIZE:2 YSIZE:2 gnPopulationsNb cells:5								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ZeroLife

  //  0	 XSIZE 1-  make					       // For each column
  //	  0  YSIZE 1-  make				       // For each row
  //	     I J Cells  0C!
  //	  endmake
  //  endmake

   ( Reset all the field )
   0 0 Cells
   ( push	  edi		   )	     [: 57 :]
   ( clr	  al		   )	     [: 32 C0 :]
   ( mov	  edi, [esi]	   )	     [: 8B 3E :]
   ( add	  esi, 4	   )	     [: 83 C6 04 :]
   ( mov	  ecx, XSIZE*YSIZE )	     [: B9 48800 :]
   ( cld			   )	     [: FC :]
   ( rep	  stosb		   )	     [: F3 AA :]
   ( pop	  edi		   )	     [: 5F :]

   ( Mark border cells as inactive {bit 7 is set} )
   0  XSIZE 1-  make
       7  I  0	       Cells  SetBit			       // First line
       7  I  YSIZE 1-  Cells  SetBit			       // Last line
   endmake
   0  YSIZE 1-  make
       7  0	    I  Cells  SetBit			       // Fist column
       7  XSIZE 1-  I  Cells  SetBit			       // Last column
   endmake

   ( Nobody is alive ! )
   gnPopulationsNb 0!

   ;  // ZeroLife


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputeEvenGeneration											        \\
// 	 Stack : ( --- )												        \\
// Description : Compute generation from EVEN generation to ODD generation.						        \\
//    Callers : ComputeNextGeneration											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ComputeEvenGeneration

   (	        push   esi			 )      [: 56 :]
   (	        mov    esi, [esi]		 )      [: 8B 36 :]
   (	        clr    ecx			 )      [: 33 C9 :]
   (	        mov    edx, 640 * 464 - 640	 )      [: BA 80 85 04 00 :]
   (	        dec    edx			 )      [: 4A :]

   (   loop:    test   byte ptr [esi], 80h	 )      [: F6 06 80 :]
   (	        jnz    @@NextCell		 )      [: 75 63 :]

   ( Compute in BL the number of neighbors for this cell )
   (		 clr    ebx			 )      [: 33 DB :]
   (		 mov    eax, [esi - 641]	 )      [: 8B 86 7F FD FF FF :]
   (		 bt     eax, 16			 )      [: 0F BA E0 10 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 bt     eax, 8			 )      [: 0F BA E0 08 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 bt     eax, 0			 )      [: 0F BA E0 00 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 mov    eax, [esi - 1]		 )      [: 8B 46 FF :]
   (		 bt     eax, 16			 )      [: 0F BA E0 10 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 bt     eax, 0			 )      [: 0F BA E0 00 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 mov    eax, [esi + 639]	 )      [: 8B 86 7F 02 00 00 :]
   (		 bt     eax, 0			 )      [: 0F BA E0 00 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 bt     eax, 8			 )      [: 0F BA E0 08 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]
   (		 bt     eax, 16			 )      [: 0F BA E0 10 :]
   (		 adc    bl, 0			 )      [: 80 D3 00 :]

   (		 cmp    bl, 2			 )      [: 80 FB 02 :]
   (		 jl     dead			 )      [: 7C 12 :]
   (		 cmp    bl, 3			 )      [: 80 FB 03 :]
   (		 jg     dead			 )      [: 7F 0D :]
   (		 je     living			 )      [: 74 05 :]
   (		 test   byte ptr [esi], 01h	 )      [: F6 06 01 :]
   (		 jz     dead			 )      [: 74 06 :]

   (   living:	 or     byte ptr [esi], 02h	 )      [: 80 0E 02 :]
   (		 inc    ecx			 )      [: 41 :]
   (		 jmp    NextCell		 )      [: EB 03 :]

   (   dead:	 and    byte ptr [esi], 0FDh	 )      [: 80 26 FD :]

   (   NextCell: inc    esi			 )      [: 46 :]
   (		 dec    edx			 )      [: 4A :]
   (		 jne    loop			 )      [: 75 94 :]
   (		 pop    esi			 )      [: 5E :]

   ( Clear previous generation bit )
   (	        push   edi		        )      [: 57 :]
   (	        push   ecx		        )      [: 51 :]
   (	        push   esi		        )      [: 56 :]
   (	        mov    ebx, [esi+4]	        )      [: 8B 5E 04 :]
   (	        mov    esi, [esi]	        )      [: 8B 36 :]
   (	        clr    ecx		        )      [: 33 C9 :]
   (	        clr    edx		        )      [: 33 D2 :]
   (	        clr    edi		        )      [: 33 FF :]
   (   ClrPrevG: mov    al, [esi]	        )      [: 8A 06 :]
   (	        and    al, 3		        )      [: 24 03 :]
   (	        jz     NoChang		        )      [: 74 10 :]
   (	        cmp    al, 3		        )      [: 3C 03 :]
   (	        je     NoChang		        )      [: 74 0C :]
   (   Change:  and    byte ptr [esi], 82h      )      [: 80 26 82 :]
   (	        or     byte ptr [esi], 04h      )      [: 80 0E 04 :]
   (	        mov    [ebx + edx*4], edx       )      [: 89 14 BB :]
   (	        inc    edi		        )      [: 47 :]
   (	        jmp    Done		        )      [: EB 03 :]
   (   NoChang: and    byte ptr [esi], 82h      )      [: 80 26 82 :]
   (   Done:    inc    esi		        )      [: 46 :]
   (	        inc    edx		        )      [: 42 :]
   (	        cmp    edx, 640*464	        )      [: 81 FA 00 88 04 00 :]
   (	        jne    ClrPrevG		        )      [: 75 DD :]
   (	        pop    esi		        )      [: 5E :]
   (	        mov    [esi+4], edi	        )      [: 89 7E 04 :]
   (	        pop    ecx		        )      [: 59 :]
   (	        mov    [esi], ecx	        )      [: 89 0E :]
   (	        pop    edi		        )      [: 5F :]

   ;  // ComputeEvenGeneration


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputeOddGeneration											        \\
// 	 Stack : ( addr1 addr2 --- n1 n2 )										        \\
// 		     <addr1> : base address of changes									        \\
// 		     <addr2> : base address of cells									        \\
// 		     <n1>    : number of cells which have changed (stored in changes array).				        \\
// 		     <n2>    : new number of cells									        \\
// Description : Compute generation from ODD generation to EVEN generation.						        \\
//    Callers : ComputeNextGeneration											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ComputeOddGeneration

   (	        push   esi		        )      [: 56 :]
   (	        mov    esi, [esi]	        )      [: 8B 36 :]
   (	        clr    ecx		        )      [: 33 C9 :]
   (	        mov    edx, 640 * 464 - 640     )      [: BA 80 85 04 00 :]
   (	        dec    edx		        )      [: 4A :]

   (   loop:    test   byte ptr [esi], 80h      )      [: F6 06 80 :]
   (	        jnz    @@NextCell	        )      [: 75 63 :]

   ( Compute in BL the number of neighbors for this cell )
   (	        clr    ebx		        )      [: 33 DB :]
   (	        mov    eax, [esi - 641]	        )      [: 8B 86 7F FD FF FF :]
   (	        bt     eax, 17		        )      [: 0F BA E0 11 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        bt     eax, 9		        )      [: 0F BA E0 09 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        bt     eax, 1		        )      [: 0F BA E0 01 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        mov    eax, [esi - 1]	        )      [: 8B 46 FF :]
   (	        bt     eax, 17		        )      [: 0F BA E0 11 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        bt     eax, 1		        )      [: 0F BA E0 01 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        mov    eax, [esi + 639]	        )      [: 8B 86 7F 02 00 00 :]
   (	        bt     eax, 1		        )      [: 0F BA E0 01 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        bt     eax, 9		        )      [: 0F BA E0 09 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]
   (	        bt     eax, 17		        )      [: 0F BA E0 11 :]
   (	        adc    bl, 0		        )      [: 80 D3 00 :]

   (	        cmp    bl, 2		        )      [: 80 FB 02 :]
   (	        jl     dead		        )      [: 7C 12 :]
   (	        cmp    bl, 3		        )      [: 80 FB 03 :]
   (	        jg     dead		        )      [: 7F 0D :]
   (	        je     living		        )      [: 74 05 :]
   (	        test   byte ptr [esi], 02h      )      [: F6 06 02 :]
   (	        jz     dead		        )      [: 74 06 :]

   (   living:  or     byte ptr [esi], 01h      )      [: 80 0E 01 :]
   (	        inc    ecx		        )      [: 41 :]
   (	        jmp    NextCell		        )      [: EB 03 :]

   (   dead:    and    byte ptr [esi], 0FEh     )      [: 80 26 FE :]

   (  NextCell: inc    esi		        )      [: 46 :]
   (	        dec    edx		        )      [: 4A :]
   (	        jne    loop		        )      [: 75 94 :]
   (	        pop    esi		        )      [: 5E :]

   ( Clear previous generation bit )
   (	        push   edi		        )      [: 57 :]
   (	        push   ecx		        )      [: 51 :]
   (	        push   esi		        )      [: 56 :]
   (	        mov    ebx, [esi+4]	        )      [: 8B 5E 04 :]
   (	        mov    esi, [esi]	        )      [: 8B 36 :]
   (	        clr    ecx		        )      [: 33 C9 :]
   (	        clr    edx		        )      [: 33 D2 :]
   (	        clr    edi		        )      [: 33 FF :]
   (  ClrPrevG: mov    al, [esi]	        )      [: 8A 06 :]
   (	        and    al, 3		        )      [: 24 03 :]
   (	        jz     NoChange		        )      [: 74 10 :]
   (	        cmp    al, 3		        )      [: 3C 03 :]
   (	        je     NoChange		        )      [: 74 0C :]
   (   Change:  and    byte ptr [esi], 81h      )      [: 80 26 81 :]
   (	        or     byte ptr [esi], 04h      )      [: 80 0E 04 :]
   (	        mov    [ebx + edx*4], edx       )      [: 89 14 BB :]
   (	        inc    edi		        )      [: 47 :]
   (	        jmp    Done		        )      [: EB 03 :]
   (  NoChange: and    byte ptr [esi], 81h      )      [: 80 26 81 :]
   (   Done:    inc    esi		        )      [: 46 :]
   (	        inc    edx		        )      [: 42 :]
   (	        cmp    edx, 640*464	        )      [: 81 FA 00 88 04 00 :]
   (	        jne    ClrPrevG		        )      [: 75 DD :]
   (	        pop    esi		        )      [: 5E :]
   (	        mov    [esi+4], edi	        )      [: 89 7E 04 :]
   (	        pop    ecx		        )      [: 59 :]
   (	        mov    [esi], ecx	        )      [: 89 0E :]
   (	        pop    edi		        )      [: 5F :]

   ;  // ComputeOddGeneration


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputeNextGeneration											        \\
// 	 Stack : ( --- n )												        \\
// Description : Compute next generation. Return the new nb of cells.							        \\
//    Callers : NextGeneration												        \\
// 	 Calls : generation# cells changes ComputeEvenGeneration ComputeOddGeneration					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputeNextGeneration
   -1 changes						       // ( --- addr1 )
   0 0 cells						       // ( addr1 --- addr1 addr2 )
   generation# @ 2 mod if
       ComputeOddGeneration
   else
       ComputeEvenGeneration
   endif
   ;  // ComputeNextGeneration


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowGeneration												        \\
// 	 Stack : ( f --- )  <f>: force all cells to be redrawn								        \\
// Description : Show cells which have changed.										        \\
//    Callers : NextGeneration Help CellsEditor									        \\
// 	 Calls : CellOffColor CellOnColor:2 generation# gnPopulationsNb InfosLineNo cells changes			        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
graphics
:  ShowGeneration
   not if

       ( Show only cells which have changed )
       dup not if
	   drop
       else
	   1- 0 swap make
	       I changes				       // ( --- x y f )
	       if CellOnColor else CellOffColor endif	       // ( x y f --- x y n )
	       setpixel					       // ( x y n --- )
	   endmake
       endif

   else

       ( Show all cells )
       int nCellsNb  gnPopulationsNb nCellsNb @!	       // Nb of cells to draw
       0  #gcols 1-  make
	   I 8 mod 0== if
	       screen
	       I 8 /  InfosLineNo  '.  !xychar
	       graphics
	   endif
	   0 #grows 17 -	 make
	       generation# @ 2 mod
	       J I cells	 GetBit	 if
		   J I CellOnColor SetPixel
		   nCellsNb 1-!
	       endif
	   endmake
	   nCellsNb @ when
       endmake

   endif
   ;  // ShowGeneration
screen


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_ShowGeneration											        \\
// 	 Stack : ( --- )												        \\
// Description : Show cells which have changed.										        \\
//    Callers :													        \\
// 	 Calls : generation# gnCornerTopLeft_X gnCornerTopLeft_Y cells							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_ShowGeneration
   ?UpdateCursor false!					       // Don't update cursor position when drawing
   0 #rows 3 -  make
       I 0 gotoxy
       0  #cols 1-  make
	   gnCornerTopLeft_X @ I +  gnCornerTopLeft_Y @ J +  Cells // ( --- addr )
	   2 over  getbit
	   if						       // ( addr b1 b2 --- addr )
	       generation# @ 2 mod  swap  GetBit	       // ( addr --- b )
	       if					       // ( b --- )
		   J I " o" !xystring
	       else
		   J I " ." !xystring
	       endif
	   else
	       drop
	   endif
       endmake
   endmake
   ?UpdateCursor true!					       // Update cursor position when drawing
   ;  // Editor_ShowGeneration


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_ShowCells											        \\
// 	 Stack : ( --- )												        \\
// Description : Show generation, in editor mode (text mode).								        \\
//    Callers : Editor_Down Editor_Up Editor_Left Editor_Right Help CellsEditor					        \\
// 	 Calls : char_cell_off char_cell_on generation# gnCornerTopLeft_X gnCornerTopLeft_Y cells			        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_ShowCells
   ?UpdateCursor false!					       // Don't update cursor position when drawing
   0 #rows 2 -  make
       0 I gotoxy
       0  #cols 1-  make
	   generation# @ 2 mod
	   gnCornerTopLeft_X @ I +  gnCornerTopLeft_Y @ J +  Cells
	   getbit if char_cell_on else char_cell_off endif
	   emit
       endmake
   endmake
   ?UpdateCursor true!					       // Update cursor position when drawing
   ;  // Editor_ShowCells


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : NextGeneration												        \\
// 	 Stack : ( --- )												        \\
// Description : Compute and show next generation of cells.								        \\
//    Callers : AutomaticMode HeartLife										        \\
// 	 Calls : generation# gnPopulationsNb InfosLineNo:2 ShowGenPop ComputeNextGeneration ShowGeneration		        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  NextGeneration
   0 InfosLineNo red 'C !xyachar
   ComputeNextGeneration				       // ( --- n1 n2) n1: nb of changed cells n2: new population
   0 InfosLineNo 0x20 !xychar
   gnPopulationsNb !					       // ( n --- ) Update the nb of cells
   generation# 1+!					       // Next generation
   ShowGenPop						       // Redraw the nb of cells and the generation nb
   FALSE ShowGeneration					       // Show only cells which have changed
   ;  // NextGeneration


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AutomaticMode												        \\
// 	 Stack : ( --- )												        \\
// Description : Enter into automatic mode.										        \\
//    Callers : HeartLife												        \\
// 	 Calls : KEY_AUTO InfosLineNo ShowAvailableKeysCmd NextGeneration						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AutomaticMode
   '~ >xylstring_char
   0 black white     >xylstring_attr
   1 black lightgray >xylstring_attr
   InfosLineNo 19 swap					       // ( --- x y )
   "   ~{0}F3~{1} cancel automatic mode "
   !xylstring clreol					       // ( x y addrz --- )
   false ilogical lStop?				       // Exit loop ?
   begin
       NextGeneration
       begin
	   lStop? false?
	   ?key
	   and
       while
	   key KEY_AUTO ==  if
	       lStop? true!
	   else
	       lStop? false!
	   endif
       repeat
       lStop? true?
   until
   ShowAvailableKeysCmd					       // Draw the available keys.
   ;   // AutomaticMode


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_ChangeCell											        \\
// 	 Stack : ( --- )												        \\
// Description : Change the Living/Death status of the current cell.							        \\
//    Callers : CellsEditor												        \\
// 	 Calls : char_cell_off char_cell_on gnPopulationsNb:2 gnEditor_X:4 gnEditor_Y:4 gnCornerTopLeft_X gnCornerTopLeft_Y     \\
// 		 GetCell:2 RvsCell ShowGenPop										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_ChangeCell

   gnEditor_X @	 gnEditor_Y @  RvsCell			       // Change the state of the cell under the cursor
   gnEditor_X @	 gnCornerTopLeft_X @  -			       // ( --- col )
   gnEditor_Y @	 gnCornerTopLeft_Y @  -			       // ( col --- col row )
   gnEditor_X @	 gnEditor_Y @  GetCell			       // ( col row --- col row n )
   if  char_cell_on  else  char_cell_off  endif
   !xychar

   ( Update the nb of cells )
   gnEditor_X @	 gnEditor_Y @  GetCell  if
       gnPopulationsNb 1+!
   else
       gnPopulationsNb 1-!
   endif

   ( Draw the nb of cells )
   ShowGenPop

   ;  // Editor_ChangeCell


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_Down												        \\
// 	 Stack : ( --- )												        \\
// Description : Into cells editor, move cursor one line down.								        \\
//    Callers : CellsEditor												        \\
// 	 Calls : YSIZE:3 YINC gnEditor_Y:4 gnCornerTopLeft_Y:3 Editor_ShowCells						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_Down
   gnEditor_Y @	 YSIZE #rows 1- - !=  if
       gnEditor_Y @  gnCornerTopLeft_Y @  -  #rows 1- 1- !=  if
	   gnEditor_Y 1+!
       else
	   YINC gnCornerTopLeft_Y +!
	   gnEditor_Y @	 YSIZE #rows 1- - >  if
	       YSIZE #rows 1- -	 gnCornerTopLeft_Y !
	   endif
	   Editor_ShowCells				       // Show cells in text mode
       endif
   endif
   ;  // Editor_Down


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_Up												        \\
// 	 Stack : ( --- )												        \\
// Description : Into cells editor, move cursor one line up.								        \\
//    Callers : CellsEditor												        \\
// 	 Calls : YINC gnEditor_Y:3 gnCornerTopLeft_Y:5 Editor_ShowCells							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_Up
   gnEditor_Y @0!= if
       gnEditor_Y @  gnCornerTopLeft_Y @  != if
	   gnEditor_Y 1-!
       else
	   gnCornerTopLeft_Y @0!=  if
	       YINC gnCornerTopLeft_Y -!
	       gnCornerTopLeft_Y @ 0< if
		   gnCornerTopLeft_Y 0!
	       endif
	       Editor_ShowCells				       // Show cells in text mode
	   endif
       endif
   endif
   ;  // Editor_Up


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_Left												        \\
// 	 Stack : ( --- )												        \\
// Description : Into cells editor, move cursor one char left.								        \\
//    Callers : CellsEditor												        \\
// 	 Calls : XINC gnEditor_X:3 gnCornerTopLeft_X:5 Editor_ShowCells							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_Left
   gnEditor_X @0!= if
       gnEditor_X @  gnCornerTopLeft_X @  !=  if
	   gnEditor_X 1-!
       else
	   gnCornerTopLeft_X @0!=  if
	       XINC gnCornerTopLeft_X -!
	       gnCornerTopLeft_X @ 0< if
		   gnCornerTopLeft_X 0!
	       endif
	       Editor_ShowCells				       // Show cells in text mode
	   endif
       endif
   endif
   ;  // Editor_Left


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Editor_Right												        \\
// 	 Stack : ( --- )												        \\
// Description : Into cells editor, move one char right.								        \\
//    Callers : CellsEditor												        \\
// 	 Calls : XSIZE:3 XINC gnEditor_X:4 gnCornerTopLeft_X:3 Editor_ShowCells						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Editor_Right
   gnEditor_X @	 XSIZE #cols - !=  if
       gnEditor_X @  gnCornerTopLeft_X @  -  #cols 1- !=  if
	   gnEditor_X 1+!
       else
	   XINC gnCornerTopLeft_X +!
	   gnEditor_X @	 XSIZE #cols - >  if
	       XSIZE #cols -  gnCornerTopLeft_X !
	   endif
	   Editor_ShowCells				       // Show cells in text mode
       endif
   endif
   ;  // Editor_Right


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowHelp_1												        \\
// 	 Stack : ( --- )												        \\
// Description : Show life help, page #1										        \\
//    Callers : ShowHelp												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowHelp_1
   black graphics clear screen
   '~ >xylstring_char
   0 black lightgray >xylstring_attr
   1 black white     >xylstring_attr
   2 black yellow    >xylstring_attr
   3 black green     >xylstring_attr
   4 black white     >xylstring_attr
   5 black yellow    >xylstring_attr
   6 black red	     >xylstring_attr

   80 fstring tittle$
   " ~{1}FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " CONWAY'S LIFE GAME - RULES" +$
   #cols over $len - 2/ 0 rot !xylstring

    0  2 " In the game of Life, cells are living on a large checkerboard and die or" !xylstring
    0  3 " reproduce  according to some simple genetic rules. First note that each cell" !xylstring
    0  4 " of the checkerboard (640x464) has ~{1}eight~{0} neighboring cells, four adjacent" !xylstring
    0  5 " orthogonally, four adjacent diagonally. The rules are:" !xylstring
    0  6 "   ~{2}Survivals~{0}: Every cell with ~{1}two~{0} or ~{1}three~{0} neighbors survives for the next" !xylstring
    0  7 "              generation." !xylstring
    0  8 "      ~{2}Deaths~{0}: Each cell with ~{1}four~{0} or more neighbors dies from" !xylstring
    0  9 "              ~{3}overpopulation~{0}. Every cell with ~{1}one~{0} neighbor or ~{1}none~{0} dies" !xylstring
    0 10 "              from ~{3}isolation~{0}." !xylstring
    0 11 "      ~{2}Births~{0}: Each empty cell adjacent to exactly ~{1}three~{0} neighbors" !xylstring
    0 12 "              (no more, no fewer) is a ~{3}birth~{0} cell." !xylstring
    0 13 " You must understand that all births and deaths occur ~{1}simultaneous~{0}. Together" !xylstring
    0 14 " they constitute a single generation. Here is an example how things work:" !xylstring

    3 16 "  gen # 1       gen # 2       gen # 3       gen # 4       gen # 5" !xylstring
    3 17 " .........     .........     .........     ...o.....     ..ooo...." !xylstring
    3 18 " .........     ...o.....     ..ooo....     ..ooo....     ..ooo...." !xylstring
    3 19 " ..ooo.... --> ..ooo.... --> .o...o... --> .o...o... --> .o...o..." !xylstring
    3 20 " ..o..o...     .oo.oo...     .oo.oo...     .oo.oo...     .oo.oo..." !xylstring
    3 21 " ..o...o..     .........     .........     .........     ........." !xylstring

    3 23 " .........     .........     .........     ...~{5}o~{0}....." !xylstring
    3 24 " .........     ...~{5}o~{0}.....     ..~{5}o~{4}o~{5}o~{0}....     ..~{4}ooo~{0}...." !xylstring
    3 25 " ..ooo....     ..~{4}ooo~{0}....     .~{5}o~{6}ooo~{5}o~{0}...     .~{4}o~{0}...~{4}o~{0}..." !xylstring
    3 26 " ..o..o...     .~{5}o~{4}o.o~{5}o...     .~{4}oo~{0}.~{4}o~{0}o...     .~{4}oo~{0}.~{4}oo~{0}..." !xylstring
    3 27 " ..o...o..     ..~{6}o~{0}...~{6}o~{0}..     .........     ........." !xylstring

    59 23 " ..~{5}o~{4}o~{5}o~{0}...." !xylstring
    59 24 " ..~{4}ooo~{0}...." !xylstring
    59 25 " .~{4}o~{0}...~{4}o~{0}..." !xylstring
    59 26 " .~{4}oo~{0}.~{4}oo~{0}..." !xylstring
    59 27 " ........." !xylstring

    3 29 " ~{4}o~{0} are survivals   ~{5}o~{0} are births   ~{6}o~{0} are deaths" !xylstring

   #cols 26 -  #rows 1- " page 1/3 - Press ~{1}ESC~{0} or ~{1}\19" !xylstring

   ;  // ShowHelp_1


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowHelp_2												        \\
// 	 Stack : ( --- )												        \\
// Description : Show life help, page #2										        \\
//    Callers : ShowHelp												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowHelp_2
   black graphics clear screen
   '~ >xylstring_char
   0 black lightgray >xylstring_attr
   1 black white     >xylstring_attr
   2 black yellow    >xylstring_attr

   80 fstring tittle$
   " ~{1}FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " CONWAY'S LIFE GAME - KEYS" +$
   #cols over $len - 2/ 0 rot !xylstring

    0  2 " There are two modes: under the editor and not. Under editor, you can edit" !xylstring
    0  3 " the cells, i.e. edit the current generation. The editor let you see and" !xylstring
    0  4 " and edit only a portion of the large checkerboard (of 640x464), this is" !xylstring
    0  5 " a window, with scrolling. If you leave editor. you can then see population" !xylstring
    0  6 " involving generations after generations." !xylstring

    0  8 " You can ~{1}save~{0}/~{1}load~{0} a generation into/from a file. This is useful to recall" !xylstring
    0  9 " amazing populations: this avoid to edit again these populations." !xylstring

    0 11 " ~{2}Under the editor" !xylstring
    0 12 "   ~{1}F1~{0}           help" !xylstring
    0 13 "   ~{1}space~{0}        toggle cell on/off" !xylstring
    0 14 "   ~{1}\18 \19 \1b \1a ~{0}    move the cursor throught cells" !xylstring
    0 15 "   ~{1}Ctrl \18 \19 \1b \1a~{0} move the cursor throught cells and put all cells on" !xylstring
    0 16 "   ~{1}Alt \18 \19 \1b \1a~{0}  move the cursor throught cells and put all cells off" !xylstring
    0 17 "   ~{1}Alt S~{0}        save current generation into a file" !xylstring
    0 18 "   ~{1}Alt L~{0}        load a generation from a file" !xylstring
    0 19 "   ~{1}F10~{0}          leave editor to compute next generations" !xylstring
    0 20 "   ~{1}ESC~{0}          exit application" !xylstring

    0 22 " ~{2}Outside the editor" !xylstring
    0 23 "   ~{1}F1~{0}           help" !xylstring
    0 24 "   ~{1}F2~{0}           compute the next generation" !xylstring
    0 25 "   ~{1}F3~{0}           automatic mode: compute next generations until ~{1}F3~{0} is pressed" !xylstring
    0 26 "   ~{1}F10~{0}          return to editor mode" !xylstring
    0 27 "   ~{1}ESC~{0}          exit application" !xylstring

   #cols 31 -  #rows 1- " page 2/3 - Press ~{1}ESC~{0} or ~{1}\18~{0} or ~{1}\19" !xylstring

   ;  // ShowHelp_2


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowHelp_3												        \\
// 	 Stack : ( --- )												        \\
// Description : Help, page #3												        \\
//    Callers : ShowHelp												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowHelp_3
   black graphics clear screen
   0 const x0
   1 const y0
   '~ >xylstring_char
   0 black lightgray >xylstring_attr
   1 black white     >xylstring_attr
   2 black yellow    >xylstring_attr

   80 fstring tittle$
   " ~{1}FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " CONWAY'S LIFE GAME - ABOUT FROTH" +$
   #cols over $len - 2/ 0 rot !xylstring

   x0 y0  2 + " This game is written in ~{1}FROTH~{0}.  This is a free implementation of" !xylstring
   x0 y0  3 + " the ~{1}FORTH~{0} language. Here are the highlights of ~{2}FROTH~{0} implementation:" !xylstring
   x0 y0  5 + " - Complete and highly interactive environment (interpret, 386 compiler," !xylstring
   x0 y0  6 + "   multi-buffers source built-in editor);" !xylstring
   x0 y0  7 + " - Run in 32-bits protected mode (flat model) under MS-DOS" !xylstring
   x0 y0  8 + "   (or DOS box under Windows 95);" !xylstring
   x0 y0  9 + " - Tools for cross-reference (update each definition header);" !xylstring
   x0 y0 10 + " - Definitions browser (instant go to any definition or variable);" !xylstring
   x0 y0 11 + " - Up to 32 running threads (cooperative multi-tasking);" !xylstring
   x0 y0 12 + " - ~{2}Automatic~{0} and ~{2}Dynamic~{0} variables (with garbage collector);" !xylstring
   x0 y0 13 + " - Fixed and Dynamic string handling;" !xylstring
   x0 y0 14 + " - Ability to build .EXE files (stand alone application under MS-DOS);" !xylstring
   x0 y0 15 + " - Complete on-line help for each cold defined word;" !xylstring
   x0 y0 16 + " - Comes with a complete 100+ pages manual: tutorial, reference, glossary." !xylstring
   x0 y0 18 + " This is a ~{1}public domain~{0} free software (executable and sources). You can" !xylstring
   x0 y0 19 + " dowload ~{2}FROTH~{0} from this FTP site:  ~{1}taygeta.com/Forth/Misc" !xylstring
   x0 y0 21 + " ~{2}FROTH~{0} is fully written in 386 assembler (Borland TASM v4)." !xylstring
   x0 y0 23 + " Please send to the author your comments, ideas, bugs report, programs, ..." !xylstring
   x0 y0 24 + " to  ~{1}singla@email.teaser.fr" !xylstring

   '~ >xylstring_char
   0 black lightgray >xylstring_attr
   1 black white     >xylstring_attr
   #cols 26 -  #rows 1- " page 3/3 - Press ~{1}ESC~{0} or ~{1}\18" !xylstring

   ;  // ShowHelp_3


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowHelp												        \\
// 	 Stack : ( --- )												        \\
// Description : Show life help.											        \\
//    Callers : Help													        \\
// 	 Calls : ShowHelp_1 ShowHelp_2 ShowHelp_3									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowHelp
   1 iint  nPageNo
   false ilogical lStop?
   begin
       nPageNo @ case
	   1 of	 ShowHelp_1  endof
	   2 of	 ShowHelp_2  endof
	   3 of	 ShowHelp_3  endof
       endcase
       false ilogical lOk?
       begin
	   key case fkeys
	       0x1B		    of  lOk? true!  lStop? true!			 endof
	       KEY_UP		    of  tipof
	       KEY_UP keypad-ed +   of  lOk? true!  nPageNo @ 1 > if  nPageNo 1-! endif	 endof
	       KEY_DOWN		    of  tipof
	       KEY_DOWN keypad-ed + of  lOk? true!  nPageNo @ 3 < if  nPageNo 1+! endif	 endof
	   endcase
	   lOk? true?
       until
       lStop? true?
   until
   ;  // ShowHelp


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Help													        \\
// 	 Stack : ( --- )												        \\
// Description : Show life help.											        \\
//    Callers : CellsEditor HeartLife											        \\
// 	 Calls : CellOffColor:2 EditorActive? InfosLineNo:2 Frame ShowGeneration Editor_ShowCells ShowHelp		        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Help
   graphics CellOffColor clear screen			       // Erase graphics screen (set with off cells)
   ShowHelp						       // Show life help
   graphics CellOffColor clear screen			       // Erase graphics screen (set with off cells)
   InfosLineNo clrline					       // Put right color for this line
   EditorActive? true? if
       Editor_ShowCells					       // Show cells in text mode
   else
       TRUE ShowGeneration				       // Show all cells, in graphics mode
       InfosLineNo clrline				       // Clear this line
   endif
   Frame						       // Redraw the 'frame' of the game
   ;  // Help


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : CellsEditor												        \\
// 	 Stack : ( --- f )  f is TRUE if BYE is asked, else FALSE							        \\
// Description : Enter into the editor of cells.									        \\
//    Callers : HeartLife:2												        \\
// 	 Calls : CellOffColor char_cell_off char_cell_on char_cell_off_curr char_cell_on_curr KEY_HELP KEY_BYE KEY_CHGCELL      \\
// 		 gnEditor_X:4 gnEditor_Y:4 gnCornerTopLeft_X gnCornerTopLeft_Y EditorActive?:2 InfosLineNo:3 GetCell:2	        \\
// 		 ShowGenPop ShowAvailableKeysCmd Frame ShowGeneration Editor_ShowCells Editor_ChangeCell Editor_Down Editor_Up  \\
// 		 Editor_Left Editor_Right Help										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  CellsEditor
   EditorActive? true!					       // Is Editor active ?
   Editor_ShowCells					       // Show cells in text mode
   ShowAvailableKeysCmd					       // Draw the availble keys.
   false ilogical lStop?				       // Flag: stop main loop ?
   false ilogical lBye?					       // Flag: bye application ?
   begin
       #cols 10 - InfosLineNo gotoXY
       '( emit  3 gnEditor_X @ .R  ', emit
       3 gnEditor_Y @ .R  ') emit			       // Print current (col, row)
       gnEditor_X @  gnCornerTopLeft_X @  -		       // ( --- x )	 Compute the current col
       gnEditor_Y @  gnCornerTopLeft_Y @  -		       // ( x --- x y )	 Compute the current row
       ddup gnEditor_X @ gnEditor_Y @ getcell		       // ( x y --- x y x y f )
       if char_cell_on_curr else char_cell_off_curr endif      // ( x y x y f --- x y x y n )
       !xychar						       // ( x y x y n --- x y )
       ddup gotoXY					       // ( x y --- x y )  Bring cursor on the current cell's position
       key						       // ( x y --- x y c )
       -rot						       // ( x y c --- c x y )
       gnEditor_X @ gnEditor_Y @ getcell		       // ( x y --- x y f )
       if char_cell_on else char_cell_off endif		       // ( x y f --- x y n )
       !xychar						       // ( x y x y n --- x y )
       FKeys case					       // ( c --- )
	   KEY_CHGCELL		 of  Editor_ChangeCell endof
	   KEY_DOWN		 of		       tipof
	   KEY_DOWN keypad-ed +	 of  Editor_Down       endof
	   KEY_UP		 of		       tipof
	   KEY_UP keypad-ed +	 of  Editor_Up	       endof
	   KEY_LEFT		 of		       tipof
	   KEY_LEFT keypad-ed +	 of  Editor_Left       endof
	   KEY_RIGHT		 of		       tipof
	   KEY_RIGHT keypad-ed + of  Editor_Right      endof
	   KEY_F10		 of  lStop? true!      endof
	   KEY_BYE		 of  lBye?  true!      endof
	   KEY_HELP		 of  Help	       endof
       endcase
       lStop? true?  lBye? true?  or
   until
   lBye? false? if
       EditorActive? false!					   // Is Editor active ?
       graphics CellOffColor clear screen			   // Erase graphics screen (set with off cells)
       InfosLineNo clrline					   // Put right color for this line
       TRUE ShowGeneration					   // Show all cells, in graphics mode
       InfosLineNo clrline					   // Clear this line
       Frame							   // Redraw the 'frame' of the game
       ShowGenPop						   // Print the generation # and the nb of populations.
       FALSE
   else
       TRUE
   endif
   ;  // CellsEditor


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HeartLife												        \\
// 	 Stack : ( --- )												        \\
// Description : The 'heart' of the game.										        \\
//    Callers : life													        \\
// 	 Calls : KEY_HELP KEY_NEXT_GEN KEY_AUTO KEY_EDIT KEY_BYE NextGeneration AutomaticMode Help CellsEditor:2	        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HeartLife
   CellsEditor						       // ( --- f ) In first, we edit cells
   not if
       false ilogical lStop?				       // Flag: stop main loop ?
       begin
	   key						       // ( --- c )
	   case
	       KEY_HELP	     of	  Help				     endof
	       KEY_NEXT_GEN  of	  NextGeneration		     endof
	       KEY_AUTO	     of	  AutomaticMode			     endof
	       KEY_EDIT	     of	  CellsEditor if lStop? true! endif  endof
	       KEY_BYE	     of	  lStop? true!			     endof
	   endcase
	   lStop? true?
       until
   endif
   ;  // HeartLife


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : CmdArgs												        \\
// 	 Stack : ( --- )												        \\
// Description : Cmd line arguments.											        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  CmdArgs
   80 fstring t$
   0  #cmdargs 1-  make
       I cmdargs$ dup $uppercase " /V" $== if
	   " LIFE v" t$ $!
	   t$ #version 10000 / <# '. hold # #> +$
	   #version 100 / 100 mod <# '. hold # # #> +$
	   #version 100 mod <# 'a + hold 0 #> +$
	   type cr
	   " Compiled with FROTH v" t$ $!
	   FrothVersion <#S> t$ $+
	   swap <# '. hold # # '. hold #> +$
	   swap 0 <# 0x20 hold swap  'a + hold #> +$
	   drop FrothDate
	   1- $month t$ $+
	   swap <# '- hold # # '- hold #> +$
	   swap <# # # # # #> +$
	   type
	   bye
       else
	   ."    Syntax: life [/?] [/V]\0d\0a"
	   ." Arguments: /? print this help\0d\0a"
	   ."            /v print the version number"
	   ." \0d\0aWARNING!  This is not the complete version: some functions are not"
	   ." \0d\0a          yet implemented:"
	   ." \0d\0a              - Save/Load a generation into/from a file"
	   ." \0d\0a              - Inside editor, move cursor with putting cells on/off"
	   bye
       endif
   endmake
   ;   // CmdArgs


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : life													        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application									        \\
//    Callers : g													        \\
// 	 Calls : C_NormalText cells changes InitThings Frame ZeroLife HeartLife						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  life
   ?turnkey if  CmdArgs	 endif				       // Cmd line arguments
   Screen C_NormalText ScreenBackground !		       // The screenbackground is Black
   GetVideoMode const nPrevVideoMode			       // Save previous video mode
   ?UpdateCursor true!					       // Update cursor when drawing
   graphics 640x480x16					       // Select graphics mode 640x480x16
   0 clear						       // Clear graphics screen
   InitThings						       // Initialize the game
   Frame						       // Print the frame
   ZeroLife						       // Empty all the cells.
   HeartLife						       // The heart of the game
   free cells						       // Free huge cells array
   free changes						       // Free another huge array
   screen GetVideoMode nPrevVideoMode != if		       // Restore previous video mode, if needed
       nPrevVideoMode SetVideoMode
   endif
   ;  // life
:  g life  ;


( Print code/data size )
( -------------------- )

infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

\ " \\froth\\examples\\life.exe" save.exe
\ cr " LIFE.EXE app size is " type . "  bytes" type

