// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: MATCHES.FTH    Author: Oliver SINGLA	 Date: Sep-10-94    Version: 3.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:														        \\
//     Game of the 23 matches: the one which take the last one loose...							        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//     3.00    Dec-27-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;

( Make SCREEN the context vocabulary )
screen

( Global variables )
int #MatchesLeft					       // Nb of matches left
int whichPlayer						       // Which player must play now ? Player or Computer
int round#						       // Round number

( Colors used )
black LightGray BackForeGround const C_NormalText	       // Normal text

( Each player )
'H const Human						       // The human player
'C const Computer					       // The computer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : build_msg												        \\
// 	 Stack : Compile-time: ( x y addrz )										        \\
// 		     Run-time: ( --- )											        \\
// Description : Create a msg, with string and position together.							        \\
//    Callers : msg_PlayFirst? msg_HumanHowManyMatches? msg_ComputerThinks msg_ComputerTook1Match msg_ComputerTook2Matches     \\
// 		 msg_ComputerTook3Matches										        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  build_msg
   build>
       0 begin  ddup + (C@)  while  1+  repeat		       // ( x y addrz --- x y addrz len )
       1+ dup 2+ ballot					       // ( x y addrz len --- x y addrz len addr )
       >R R 2+ -rot					       // ( x y addrz len addr --- x y addr addrz len )
       (cmove)						       // ( x y addr addrz len --- x y )
       R 1+ C!						       // ( x y --- x )
       R>   C!						       // ( x --- )
   does>
       dup C@ swap 1+					       // ( addr --- x addr+1 )
       dup C@ swap 1+					       // ( x addr+1 --- x y addr+2 )
       !xystring
       clreol
   ;   // build_msg

0 10 " Do you want to play first (Y/N) ? "                     build_msg msg_PlayFirst?
0 10 " How many matches do you take (1, 2 or 3) ? "            build_msg msg_HumanHowManyMatches?
0 12 " My turn... I'm thinking..."                             build_msg msg_ComputerThinks
0 12 " Computer took one match."                               build_msg msg_ComputerTook1Match
0 12 " Computer took two matches."                             build_msg msg_ComputerTook2Matches
0 12 " Computer took three matches."                           build_msg msg_ComputerTook3Matches
0 14 " Congratulations, you win this game..."                  build_msg msg_HumanWon
0 14 " Sorry, you loose this game..."                          build_msg msg_ComputerWon
0 16 " Do you want to play an other game (Y/N) ? "             build_msg msg_AnotherGame?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : doTittle												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Tittle													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  doTittle
   dup $len const len
   #cols  len 4 + -  2/	 const  x
   x 0 ' !xychar  x 1+ 0 len 2+ ' !xynchar  x len + 3 + 0 ' !xychar
   x 1 "   " !xystring  x 2+ 1 rot !xystring  x len + 2+ 1 "  " !xystring
   x 2 ' !xychar  x 1+ 2 len 2+ ' !xynchar  x len + 3 + 2 ' !xychar
   ;   // doTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : tittle													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : drawFrame												        \\
// 	 Calls : doTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Tittle
   80 fstring tittle$
   " FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # # # #> +$
   " 23 MATCHES GAME" +$
   doTittle
   ;   // Tittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Rules													        \\
// 	 Stack : ( --- )												        \\
// Description : Print the rules of the game.										        \\
//    Callers : drawFrame												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Rules
   7 5 " Rules : There is a stack of 23 matches. You play against Computer." !xystring
   7 6 "    Each one on its turn,  you will take  1,  2 or  3 matches." !xystring
   7 7 "         The one who take the last match loose the game !"           !xystring
   ;  // Rules


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : drawFrame												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame (tittle + rules) of the game.								        \\
//    Callers : Matches												        \\
// 	 Calls : Tittle Rules												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  drawFrame
   Tittle						       // Draw tittle game
   Rules						       // Draw the rules of the game
   ;   // drawFrame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PlayFirst?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Ask the question 'Play first ? '.									        \\
//    Callers : Matches												        \\
// 	 Calls : whichPlayer Human Computer msg_PlayFirst?								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PlayFirst?
   msg_PlayFirst?					       // "Play first (Y/N) ? "
   clreol						       // Clear until End Of Line (clear previous msg)
   true cursor
   begin
       key
       charuppercase
       dup  'Y !=
       over 'N != and
   while
       drop
   repeat
   false cursor
   dup emit						       // Show char typed

   ( Initialize which player play first )
   'Y == if Human else Computer endif whichPlayer !

   ;   // PlayFirst?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawMatch												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw a not burned match, at location (col, row).							        \\
//    Callers : InstallMatches												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawMatch
      ddup  red	   220 !xyachar
   1+ ddup  yellow 219 !xyachar
   1+ ddup  yellow 219 !xyachar
   1+ ddup  yellow 219 !xyachar
   ddrop
   ;   // DrawMatch


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawBurnedMatch											        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw a burned match, at location (col, row).								        \\
//    Callers : RemoveMatches												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawBurnedMatch
      ddup  DarkGray  220 !xyachar
   1+ ddup  brown     219 !xyachar
   1+ ddup  brown     219 !xyachar
   1+ ddup  brown     219 !xyachar
   ddrop
   ;   // DrawBurnedMatch


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InstallMatches												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Install the initial matches to burn, starting at location (col, row).					        \\
//    Callers : InitMatches PlayGame											        \\
// 	 Calls : DrawMatch												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InstallMatches
   0 22 make
       over i 3 * +   over  DrawMatch
   endmake
   ddrop
   ;   // InstallMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : RemoveMatches												        \\
// 	 Stack : ( n --- )												        \\
// Description : Remove matches took by any player from screen and memory.						        \\
//    Callers : ComputerPlay HumanPlay											        \\
// 	 Calls : #MatchesLeft:3 DrawBurnedMatch										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  RemoveMatches  ( n --- )
   dup
   6 18 rot
   #MatchesLeft @ swap -  #MatchesLeft @ 1-  swap  -1  +make
       over i 3 * +  over  DrawBurnedMatch
   endmake
   ddrop
   #MatchesLeft -!					       // Update the number of matches left
   ;   // RemoveMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputerThinks												        \\
// 	 Stack : ( --- n )												        \\
// Description : The computer choose how many matches he will take (1, 2 or 3).						        \\
//    Callers : ComputerPlay												        \\
// 	 Calls : #MatchesLeft:2												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputerThinks
   ( Try to leave (1 + 4.n) matches )
   true
   1 3 make
       #MatchesLeft @ i - 1- 4 mod 0==
       if
	   drop i false false
       else
	   true
       endif
       when
   endmake
   if
       1  3 #MatchesLeft @ min  rnd
   endif
   ;   // Computerthinks


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputerPlay												        \\
// 	 Stack : ( --- )												        \\
// Description : The play of the computer.										        \\
//    Callers : PlayGame												        \\
// 	 Calls : msg_ComputerThinks msg_ComputerTook1Match msg_ComputerTook2Matches msg_ComputerTook3Matches RemoveMatches      \\
// 		 ComputerThinks												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputerPlay
   msg_ComputerThinks					       // "Computer is thinking"
   750 delay						       // 3/4 sec of pseudo-reflection
   ComputerThinks					       // The computer choose how many matches he burn
   dup RemoveMatches					       // Remove the matches took by computer
   case
       1 of  msg_ComputerTook1Match    endof
       2 of  msg_ComputerTook2Matches  endof
       3 of  msg_ComputerTook3Matches  endof
   endcase
   clreol
   ;   // ComputerPlay


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ?under													        \\
// 	 Stack : ( n ff --- ff )											        \\
// Description : Do UNDER if false flag on top stack, else do nothing.							        \\
//    Callers : HumanPlay AnotherGame?											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ?under
   dup not if under endif
   ;   // ?under


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HumanPlay												        \\
// 	 Stack : ( --- )												        \\
// Description : The play of the human player.										        \\
//    Callers : PlayGame												        \\
// 	 Calls : #MatchesLeft msg_HumanHowManyMatches? RemoveMatches ?under						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HumanPlay
   msg_HumanHowManyMatches?
   clreol
   true cursor
   begin
       key charuppercase				       // ( --- c )
       '0 -						       // ( c1 --- c2 )
       dup 1 3 >=<=					       // ( n --- n f )	     Between 1 and 3
       over #MatchesLeft @ <= and			       // ( n f1 --- n f2 )  No more than left
       ?under						       // if f2 == 0 : ( c 0 --- 0 )
   until
   false cursor
   RemoveMatches					       // Remove the matches took by human
   ;   // HumanPlay


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Diagnostic												        \\
// 	 Stack : ( --- )												        \\
// Description : After the end of a game, congratule the winner.							        \\
//    Callers : PlayGame												        \\
// 	 Calls : whichPlayer Human msg_HumanWon msg_ComputerWon								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Diagnostic
   whichPlayer @  Human	 ==
   if
       msg_HumanWon					       // " Congratulations, you win this game..."
   else
       msg_ComputerWon					       // " Sorry, you loose this game..."
   endif
   ;   // Diagnostic


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitMatches												        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize the nb of matches on start (23), then draw them.						        \\
//    Callers : Matches												        \\
// 	 Calls : #MatchesLeft InstallMatches										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InitMatches
   23 #MatchesLeft !					       // Initialize the initial nb of matches
   6 18 InstallMatches					       // Draw the initial matches
   ;   // InitMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PlayGame												        \\
// 	 Stack : ( --- )												        \\
// Description : Play a complete game, until there is a winner (no match left on the stack)				        \\
//    Callers : Matches												        \\
// 	 Calls : #MatchesLeft:2 whichPlayer:2 Human:2 Computer InstallMatches ComputerPlay HumanPlay Diagnostic		        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PlayGame
   23 #MatchesLeft !					       // Initialize the initial nb of matches
   6 18 InstallMatches					       // Draw the initial matches
   begin
       whichPlayer @ Human == if
	   HumanPlay
	   Computer
       else
	   ComputerPlay
	   Human
       endif
       whichPlayer !					       // Next player to play
       #MatchesLeft @0==
   until
   Diagnostic						       // Congratule the winner !
   ;   // PlayGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AnotherGame?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Ask if an other game is wanted ?									        \\
//    Callers : Matches												        \\
// 	 Calls : msg_AnotherGame? ?under										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AnotherGame?
   msg_AnotherGame?					       // "Do you want to play an other game (Y/N) ? "
   begin
       key charuppercase				       // ( --- c )
       dup 'Y ==  over 'N ==  or			       // ( c --- c f )
       ?under						       // if f == 0 : ( c 0 --- 0 )
   until
   16 clrline						       // Clear this line
   14 clrline						       // Clear the line where diagnostic was shown
   12 clrline						       // Clear the line where computer talks
   10 clrline						       // Clear the line where human talks
   ;   // AnotherGame?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Matches												        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application.									        \\
//    Callers :													        \\
// 	 Calls : round#:2 C_NormalText drawFrame PlayFirst? InitMatches PlayGame AnotherGame?				        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Matches
   GetVideoMode const prevVideoMode			       // Get and save previous current video mode
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   C_NormalText ScreenBackground !			       // The screenbackground is Black
   ?UpdateCursor true!					       // Update cursor when drawing
   cls							       // Clear screen
   round# 0!						       // Initialize the 1st round
   drawFrame						       // Draw the 'frame' of the game
   begin
       InitMatches					       // Install and draw the 23 initial matches
       PlayFirst?					       // Ask question 'Play first ?'
       round# 1+!					       // Next round
       PlayGame						       // Play a game
       AnotherGame? 'N ==				       // Ask if an other game ?
   until
   prevVideoMode GetVideoMode != if			       // Restore previous video mode if change
       prevVideoMode SetVideoMode			       // ( n --- ) Restore previous current video mode
   endif
   ;   // Matches


( Print informations about Code/Data size )
( --------------------------------------- )

infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

\ " \\froth\\examples\\matches.exe" save.exe
\ cr " MATCHES.EXE app size is " type . "  bytes" type

