// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: NIM.FTH        Author: Oliver SINGLA	 Date: Oct,30,94    Version: 3.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 NIM game.												        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//     3.00    Dec-27-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;

( Make SCREEN the context vocabulary )
screen

( Global variables )
int #stacks						       // Nb of stacks
int whichPlayer						       // Which player must play now ? Player or Computer
int round#						       // Round number

( Colors used )
black LightGray BackForeGround const C_NormalText	       // Normal text

( Each player )
'H const Human						       // The human player
'C const Computer					       // The computer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : build_msg												        \\
// 	 Stack : Compile-time: ( x y addrz )										        \\
// 		     Run-time: ( --- )											        \\
// Description : Create a msg, with string and position together.							        \\
//    Callers : msg_PlayFirst? msg_HumanSelectStack msg_AskNbMatches msg_ComputerThinks msg_ComputerHasPlayed msg_HumanLoose   \\
// 		 msg_ComputerLoose											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  build_msg
   build>
       0 begin  ddup + (C@)  while  1+  repeat		       // ( x y addrz --- x y addrz len )
       1+ dup 2+ ballot					       // ( x y addrz len --- x y addrz len addr )
       >R R 2+ -rot					       // ( x y addrz len addr --- x y addr addrz len )
       (cmove)						       // ( x y addr addrz len --- x y )
       R 1+ C!						       // ( x y --- x )
       R>   C!						       // ( x --- )
   does>
       dup C@ swap 1+					       // ( addr --- x addr+1 )
       dup C@ swap 1+					       // ( x addr+1 --- x y addr+2 )
       !xystring
       clreol
   ;   // build_msg

0  7 " Do you want to play first (Y/N) ? "                     build_msg msg_PlayFirst?
0  7 " Select the stack where you will remove matches."        build_msg msg_HumanSelectStack
0  7 " How many matches do you burn ? "                        build_msg msg_AskNbMatches
0  9 " Computer is thinking..."                                build_msg msg_ComputerThinks
0  9 " Computer has played..."                                 build_msg msg_ComputerHasPlayed
0 11 " I'm sorry, you loose the game !"                        build_msg msg_HumanLoose
0 11 " Congratulations ! You win computer !"                   build_msg msg_ComputerLoose
0 12 " Do you want to play an other game (Y/N) ? "             build_msg msg_AnotherGame?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : build_stack												        \\
// 	 Stack : Compile-time: ( --- )											        \\
// 		     Run-time: ( n --- addr )										        \\
// Description : Build the game's representation (stacks of matches).							        \\
//    Callers : stacks													        \\
// 	 Calls : #stacks												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  build_stack
   build>
       #stacks @  allot	 drop
   does>
       swap WordSize* +
   ;   // build_stack


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : stacks													        \\
// 	 Stack : ( n --- addr )												        \\
// Description : Current stacks state.											        \\
//    Callers : InstallMatches RemoveMatches:2 isSituationWinning?:3 ComputerThinks:3 HumanSelectStack HumanGiveNbMatches      \\
// 		 HumanPlay												        \\
// 	 Calls : build_stack												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
dynamic build_stack stacks


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PositionMatch												        \\
// 	 Stack : ( n1 n2 --- x y )    <n1>: stack #   <n2>: match #	  <x,y> screen position				        \\
// Description : Compute screen position for a given match.								        \\
//    Callers : InstallMatches RemoveMatches										        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PositionMatch
   3 * 15 +
   swap 3 * 13 +
   ;   // PositionMatch


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PositionStack												        \\
// 	 Stack : ( n --- x y )	  <n>: stack #	       <x,y> screen position						        \\
// Description : Compute screen position for a given match.								        \\
//    Callers : InstallMatches HighlightStack:8									        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PositionStack
   3 *  13 +
   ;   // PositionStack


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : doTittle												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Tittle													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  doTittle
   dup $len const len
   #cols  len 4 + -  2/	 const  x
   x 0 ' !xychar  x 1+ 0 len 2+ ' !xynchar  x len + 3 + 0 ' !xychar
   x 1 "   " !xystring  x 2+ 1 rot !xystring  x len + 2+ 1 "  " !xystring
   x 2 ' !xychar  x 1+ 2 len 2+ ' !xynchar  x len + 3 + 2 ' !xychar
   ;   // doTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Tittle													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Frame													        \\
// 	 Calls : doTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Tittle
   80 fstring tittle$
   " FROTH v" tittle$ $!
   FrothVersion <#S> tittle$ $+
   swap <# '. hold # # '. hold #> +$
   swap 0 <# 0x20 hold swap  'a + hold #> +$
   drop FrothDate
   1- $month tittle$ $+
   swap <# '- hold # # '- hold #> +$
   swap <# 0x20 hold '- hold 0x20 hold # # #> +$
   " NIM GAME" +$ doTittle
   ;   // Tittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Rules													        \\
// 	 Stack : ( --- )												        \\
// Description : Print the rules of the game.										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Rules
   3 3 " Rules : There are four stacks of matches. You play against computer."   !xystring
   3 4 "    Each one on its turn, you will take any number of matches in a" !xystring
   3 5 "         given stack. The one who takes the last match win the game !"   !xystring
   ;   // Rules


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Frame													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame (tittle + rules) of the game.								        \\
//    Callers : Nim													        \\
// 	 Calls : Tittle Rules												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Frame
   Tittle						       // Draw tittle game
   Rules						       // Draw the rules of the game
   ;   // Frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PlayFirst?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Ask the question 'Play first ? '.									        \\
//    Callers : Nim													        \\
// 	 Calls : whichPlayer Human Computer msg_PlayFirst?								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PlayFirst?
   msg_PLayFirst?					       // "Play first ?"
   true cursor
   begin
       key charuppercase
       dup  'Y !=  over 'N !=  and
   while
       drop
   repeat
   false cursor
   dup emit						       // Show char typed

   ( Initialize which player play first )
   'Y == if Human else Computer endif whichPlayer !

   ;   // PlayFirst?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawMatch												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw a not burned match, at location (col, row).							        \\
//    Callers : InstallMatches												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawMatch
      ddup  red	   220 !xyachar
   1+ ddup  yellow 219 !xyachar
   1+	    yellow 219 !xyachar
   ;   // DrawMatch


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawBurnedMatch											        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw a burned match, at location (col, row).								        \\
//    Callers : RemoveMatches												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawBurnedMatch
      ddup  DarkGray  220 !xyachar
   1+ ddup  brown     219 !xyachar
   1+	    brown     219 !xyachar
   ;   // DrawBurnedMatch


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InstallMatches												        \\
// 	 Stack : ( --- )												        \\
// Description : Install the initial matches to burn, starting at location (col, row).					        \\
//    Callers : InitGame												        \\
// 	 Calls : #stacks stacks PositionMatch PositionStack DrawMatch							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InstallMatches
   #stacks @  0	 do
       2  I PositionStack 1+  " Stack # " !xystring  I .
       I stacks @  0  do
	   J I PositionMatch
	   DrawMatch
       enddo
   enddo
   ;   // InstallMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : RemoveMatches												        \\
// 	 Stack : ( n1 n2 --- )												        \\
// Description : Remove matches took by any player from screen and memory: stack <n1>, <n2> matches.			        \\
//    Callers : ComputerPlays HumanPlay										        \\
// 	 Calls : stacks:2 PositionMatch DrawBurnedMatch									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  RemoveMatches  ( n1 n2 --- )

   ( Remove from screen )
   over stacks @ >R
   R over -  R> 1-  make
       over  I  PositionMatch  DrawBurnedMatch
   endmake

   ( Remove from memory )
   swap stacks -!

   ;   // RemoveMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : isSituationWinning?											        \\
// 	 Stack : ( stack NbMatches --- )										        \\
// Description : Look if a given situation will be winning.								        \\
//    Callers : ComputerThinks												        \\
// 	 Calls : #stacks stacks:3											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  isSituationWinning?
   over stacks  over swap  -!
   0
   0  2	 make
       0
       0   #stacks @   make
	   J  I 1- stacks  getbit			       // ( n --- n f )
	   +						       // ( n1 --- n2 )
       endmake
       2 mod						       // ( n --- f )
       +
   endmake
   0==
   -rot							       // ( stack nb f --- f stack nb )
   over stacks  over swap +!				       // ( f stack nb --- f stack nb )
   rot							       // ( f stack nb --- stack nb f )
   ;   // isSituationWinning?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputerThinks												        \\
// 	 Stack : ( --- n1 n2 )												        \\
// Description : The computer choose how many matches he will take: stack <n1>, <n2> matches.				        \\
//    Callers : ComputerPlays												        \\
// 	 Calls : #stacks:2 stacks:3 isSituationWinning?									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputerThinks
   FALSE ilogical lWinning?				       // Flag to see if a winning situation has been found

   ( Try to found a giving situation )
   1  1000  make
       0  #stacks @ 1-  rnd				       // ( --- n )  Choose randomly a stack
       dup stacks @0== if
	   drop FALSE					       // Stack is empty
       else
	   1  over stacks @  rnd			       // ( n1 --- n1 n2 ) Choose randomly a number of matches burned
	   isSituationWinning?				       // ( n1 n2 --- n1 n2 f )
	   if
	       lWinning? true! TRUE
	   else
	       ddrop FALSE
	   endif
       endif
       notwhen
   endmake

   ( If not giving situation, try randomly )
   lWinning? false? if
       begin
	   0  #stacks @ 1-  rnd				       // ( --- n )	   Choose randomly a stack
	   0  over stacks @  rnd			       // ( n1 --- n1 n2 ) Choose randomly a nb of matches
	   dup if
	       TRUE
	   else
	       ddrop FALSE
	   endif
       until
   endif

   ;   // Computerthinks


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ComputerPlays												        \\
// 	 Stack : ( --- )												        \\
// Description : The play of the computer.										        \\
//    Callers : Game													        \\
// 	 Calls : msg_ComputerThinks msg_ComputerHasPlayed RemoveMatches ComputerThinks					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ComputerPlays
   msg_ComputerThinks					       // "Computer is thinking..."
   ComputerThinks					       // The computer choose how many matches he burn
   RemoveMatches					       // ( n1 n2 --- )
   msg_ComputerHasPlayed				       // "Computer has played..."
   ;   // ComputerPlays


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ?under													        \\
// 	 Stack : ( n ff --- ff )											        \\
// Description : Do UNDER if false flag on top stack, else do nothing.							        \\
//    Callers : AnotherGame?												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ?under
   dup not if under endif
   ;   // ?under


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ?dup													        \\
// 	 Stack : ( ff --- ff )												        \\
// 		 ( tf --- tf tf )											        \\
// Description : Do DUP if tf, else do nothing.										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ?dup
   dup if dup endif
   ;   // ?dup


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HighlightStack												        \\
// 	 Stack : ( n f --- )												        \\
// Description : Surround or not a given stack tittle #.								        \\
//    Callers : HumanSelectStack:3 HumanPlay										        \\
// 	 Calls : PositionStack:8											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HighlightStack
   if
        1 over PositionStack    " Ŀ" !xystring
        1 over PositionStack 1+ ' !xychar
       11 over PositionStack 1+ ' !xychar
        1 swap PositionStack 2+ " " !xystring
   else
        1 over PositionStack    "            " !xystring
        1 over PositionStack 1+ 32 !xychar
       11 over PositionStack 1+ 32 !xychar
        1 swap PositionStack 2+ "            " !xystring
   endif
   ;   // HighlightStack


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HumanSelectStack											        \\
// 	 Stack : ( n1 --- n2 )												        \\
// Description : Human select the stack where to play.									        \\
//    Callers : HumanPlay												        \\
// 	 Calls : #stacks:3 msg_HumanSelectStack stacks HighlightStack:3							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HumanSelectStack
   msg_HumanSelectStack					       // "Select stack."
   FALSE ilogical lStop?				       // Boolean: selection made ?
   begin
       dup TRUE HighlightStack
       key
       over FALSE HighlightStack
       FKEYS
       dup case
	   13		        of   drop  lStop? true! endof
	   KEY_DOWN	        of		        tipof
	   KEY_DOWN keypad-ed + of   drop  1+	        endof
	   KEY_UP	        of		        tipof
	   KEY_UP   keypad-ed + of   drop  1-	        endof
	   '0 - dup  0  #stacks @  >=<  not if
	       drop
	   else
	       dup stacks @0!= if
		   under
	       else
		   drop
	       endif
	   endif
       endcase
       SCREEN
       dup -1 == if  drop #stacks @ 1-  endif
       #stacks @ mod
       lStop? true?
   until
   dup TRUE HighlightStack
   ;   // HumanSelectStack


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HumanGiveNbMatches											        \\
// 	 Stack : ( n1 --- n1 n2 tf ) if <CR>	 ( n1 --- n1 ff ) if <ESC>						        \\
// Description : Ask for the number of matches.										        \\
//    Callers : HumanPlay												        \\
// 	 Calls : msg_AskNbMatches stacks										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HumanGiveNbMatches
   msg_AskNbMatches
   FALSE ilogical lExit?				       // Exit loop ?
   begin
       key dup 0x1B == if				       // ( --- c )
	   drop						       // ( c --- )
	   lExit? true!	 FALSE
       else
	   '0 -						       // ( n1 c --- n1 n2 )
	   over stacks @  over 1 rot			       // ( n1 --- n1 n2 n3 1 n4 )
	   >=<= if					       // ( n1 n2 n3 1 n4 --- n1 n2 f )
	       dup '0 + emit
	       lExit? true! TRUE
	   else
	       drop
	   endif
       endif
       lExit? true?
   until
   ;   // HumanGiveNbMatches


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : HumanPlay												        \\
// 	 Stack : ( --- )												        \\
// Description : The play of the human player.										        \\
//    Callers : Game													        \\
// 	 Calls : stacks RemoveMatches HighlightStack HumanSelectStack HumanGiveNbMatches				        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  HumanPlay
   0 begin  dup stacks @0==  while  1+  repeat		       // Found the first not empty stack
   begin
       HumanSelectStack					       // ( --- n )
       HumanGiveNbMatches				       // ( n1 --- n1 n2 tf ) or ( n1 --- n1 ff )
   until
   over FALSE HighlightStack				       // Remove the box around the stack
   RemoveMatches					       // ( n1 n2 --- )
   7 ClrLine						       // Clear the line where human talks
   ;   // HumanPlay


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Diagnostic												        \\
// 	 Stack : ( --- )												        \\
// Description : After the end of a game, congratule the winner.							        \\
//    Callers : Game													        \\
// 	 Calls : whichPlayer Human msg_HumanLoose msg_ComputerLoose							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Diagnostic
   whichPlayer @  Human	 == if
       msg_HumanLoose					       // "Sorry, you loose."
   else
       msg_ComputerLoose				       // "Congratulations, you win."
   endif
   ;   // Diagnostic


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitGame												        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize the begin of a new game.									        \\
//    Callers : Nim													        \\
// 	 Calls : #stacks stacks:5 InstallMatches									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InitGame
   4 #stacks !						       // Initialize the nb of stack
   alloc stacks

   ( Initialize nb of matches in each stack )
   1 0 stacks !
   3 1 stacks !
   5 2 stacks !
   7 3 stacks !

   InstallMatches					       // Drwz initial stacks of matches

   ;   // InitGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Game													        \\
// 	 Stack : ( --- )												        \\
// Description : Play a complete game, until there is a winner (no match left on the stack)				        \\
//    Callers : Nim													        \\
// 	 Calls : #stacks whichPlayer:2 Human:2 Computer stacks:2 ComputerPlays HumanPlay Diagnostic			        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Game
   begin
       whichPlayer @ Human == if
	   HumanPlay
	   Computer
       else
	   ComputerPlays
	   Human
       endif
       whichPlayer !					       // Next player to play
       0  0  #stacks @ 1-  make	 I stacks @  +  endmake	       // ( --- n )  Total nb of matches
       0==
   until
   Diagnostic						       // Congratule the winner !
   free stacks
   ;   // Game


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AnotherGame?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Ask if an other game is wanted ? Return TRUE is yes							        \\
//    Callers : Nim													        \\
// 	 Calls : msg_AnotherGame? ?under										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AnotherGame?
   msg_AnotherGame?					       // "Play another game ?"
   begin
       key charuppercase				       // ( --- c )
       dup 'Y ==  over 'N ==  or			       // ( c --- c f )
       ?under						       // if f == 0 : ( c 0 --- 0 )
   until
   'Y ==						       // ( c --- f )
   12 clrline						       // Clear this line
   11 clrline						       // Clear the line where diagnostic was shown
    9 clrline						       // Clear the line where computer talks
    7 clrline						       // Clear the line where human talks
   ;   // AnotherGame?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Nim													        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application.									        \\
//    Callers :													        \\
// 	 Calls : round# C_NormalText Frame PlayFirst? InitGame Game AnotherGame?					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Nim
   GetVideoMode const prevVideoMode			       // Get and save previous current video mode
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   C_NormalText ScreenBackground !			       // The screenbackground is Black
   ?UpdateCursor true!					       // Update cursor when drawing
   cls							       // Clear screen
   round#  0!						       // Initialize the 1st round
   Frame						       // Draw the 'frame' of the game
   begin
       InitGame						       // Initialize a new game
       PlayFirst?					       // Ask question 'Play first ?'
       Game						       // Play a game
       AnotherGame? not					       // Ask if an other game ?
   until
   prevVideoMode GetVideoMode != if			       // Restore previous video mode if change
       prevVideoMode SetVideoMode			       // ( n --- ) Restore previous current video mode
   endif
   ;   // Nim


( Print informations about Code/Data size )
( --------------------------------------- )

infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

\ " \\froth\\examples\\nim.exe" save.exe
\ cr " NIM.EXE app size is " type . "  bytes" type

